@@ -1,4 +1,4 @@
-/*	$KAME: ipsec_dump_policy.c,v 1.13 2002/06/27 14:35:11 itojun Exp $	*/
+/* $Id: ipsec_dump_policy.c,v 1.7 2004/10/29 16:37:03 ludvigm Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, 1998, and 1999 WIDE Project.
@@ -29,12 +29,20 @@
  * SUCH DAMAGE.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 
 #include <netinet/in.h>
-#include <netinet/ipsec.h>
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else
+#  include <netinet/ipsec.h>
+#endif
 
 #include <arpa/inet.h>
 
@@ -77,6 +85,12 @@ ipsec_dump_policy(policy, delimiter)
 	char isrbuf[1024];
 	char *newbuf;
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	int32_t priority_offset;
+	char *priority_str;
+	char operator;
+#endif
+
 	/* sanity check */
 	if (policy == NULL)
 		return NULL;
@@ -89,11 +103,53 @@ ipsec_dump_policy(policy, delimiter)
 	if (delimiter == NULL)
 		delimiter = " ";
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	if (xpl->sadb_x_policy_priority == 0)
+	{
+		priority_offset = 0;
+		priority_str = "";
+	}
+	/* find which constant the priority is closest to */
+	else if (xpl->sadb_x_policy_priority < 
+	         (u_int32_t) (PRIORITY_DEFAULT / 4) * 3)
+	{
+		priority_offset = xpl->sadb_x_policy_priority - PRIORITY_HIGH;
+		priority_str = "prio high";
+	}
+	else if (xpl->sadb_x_policy_priority >= 
+	         (u_int32_t) (PRIORITY_DEFAULT / 4) * 3 &&
+	         xpl->sadb_x_policy_priority < 
+	         (u_int32_t) (PRIORITY_DEFAULT / 4) * 5)
+	{
+		priority_offset = xpl->sadb_x_policy_priority - PRIORITY_DEFAULT;
+		priority_str = "prio def";
+	}
+	else
+	{
+		priority_offset = xpl->sadb_x_policy_priority - PRIORITY_LOW;
+		priority_str = "prio low";
+	}
+
+	/* fix sign to match the way it is input */
+	priority_offset *= -1;
+	if (priority_offset < 0)
+	{
+		operator = '-';
+		priority_offset *= -1;
+	}
+	else
+	{
+		operator = '+';
+	}
+#endif
+	
 	switch (xpl->sadb_x_policy_dir) {
 	case IPSEC_DIR_ANY:
 	case IPSEC_DIR_INBOUND:
 	case IPSEC_DIR_OUTBOUND:
+#ifdef HAVE_POLICY_FWD
 	case IPSEC_DIR_FWD:
+#endif
 		break;
 	default:
 		__ipsec_errcode = EIPSEC_INVAL_DIR;
@@ -114,6 +170,11 @@ ipsec_dump_policy(policy, delimiter)
 
 	buflen = strlen(ipsp_dir_strs[xpl->sadb_x_policy_dir])
 		+ 1	/* space */
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+		+ strlen(priority_str)
+		+ ((priority_offset != 0) ? 13 : 0) /* [space operator space int] */
+		+ ((strlen(priority_str) != 0) ? 1 : 0) /* space */
+#endif
 		+ strlen(ipsp_policy_strs[xpl->sadb_x_policy_type])
 		+ 1;	/* NUL */
 
@@ -121,8 +182,29 @@ ipsec_dump_policy(policy, delimiter)
 		__ipsec_errcode = EIPSEC_NO_BUFS;
 		return NULL;
 	}
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	if (priority_offset != 0)
+	{
+		snprintf(buf, buflen, "%s %s %c %u %s", 
+	    	ipsp_dir_strs[xpl->sadb_x_policy_dir], priority_str, operator, 
+			priority_offset, ipsp_policy_strs[xpl->sadb_x_policy_type]);
+	}
+	else if (strlen (priority_str) != 0)
+	{
+		snprintf(buf, buflen, "%s %s %s", 
+	    	ipsp_dir_strs[xpl->sadb_x_policy_dir], priority_str, 
+			ipsp_policy_strs[xpl->sadb_x_policy_type]);
+	}
+	else
+	{
+		snprintf(buf, buflen, "%s %s", 
+	    	ipsp_dir_strs[xpl->sadb_x_policy_dir],
+			ipsp_policy_strs[xpl->sadb_x_policy_type]);
+	}
+#else
 	snprintf(buf, buflen, "%s %s", ipsp_dir_strs[xpl->sadb_x_policy_dir],
 	    ipsp_policy_strs[xpl->sadb_x_policy_type]);
+#endif
 
 	if (xpl->sadb_x_policy_type != IPSEC_POLICY_IPSEC) {
 		__ipsec_errcode = EIPSEC_NO_ERROR;
