@@ -1,4 +1,4 @@
-/* $Id: crypto_openssl.c,v 1.40 2004/12/29 14:23:08 vanhu Exp $ */
+/* $Id: crypto_openssl.c,v 1.40.2.2 2005/04/21 08:57:16 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -153,7 +153,7 @@ eay_str2asn1dn(str, len)
 			if (!X509_NAME_add_entry_by_txt(name, field,
 					(value[0] == '*' && value[1] == 0) ? 
 						V_ASN1_PRINTABLESTRING : MBSTRING_ASC,
-					value, -1, -1, 0)) {
+					(unsigned char *) value, -1, -1, 0)) {
 				plog(LLV_ERROR, LOCATION, NULL, 
 				     "Invalid DN field: %s=%s\n",
 				     field, value);
@@ -179,7 +179,7 @@ eay_str2asn1dn(str, len)
 	if (!X509_NAME_add_entry_by_txt(name, field,
 			(value[0] == '*' && value[1] == 0) ? 
 				V_ASN1_PRINTABLESTRING : MBSTRING_ASC,
-			value, -1, -1, 0)) {
+			(unsigned char *) value, -1, -1, 0)) {
 		plog(LLV_ERROR, LOCATION, NULL, 
 		     "Invalid DN field: %s=%s\n",
 		     field, value);
@@ -238,7 +238,7 @@ eay_hex2asn1dn(const char *hex, int len)
 	}
 	binbuf = ret->v;
 
-	BN_bn2bin(bn, binbuf);
+	BN_bn2bin(bn, (unsigned char *) binbuf);
 
 out:
 	BN_free(bn);
@@ -588,7 +588,7 @@ eay_get_x509asn1subjectname(cert)
 	int len;
 	int error = -1;
 
-	bp = cert->v;
+	bp = (unsigned char *) cert->v;
 
 	x509 = mem2x509(cert);
 	if (x509 == NULL)
@@ -600,7 +600,7 @@ eay_get_x509asn1subjectname(cert)
 	if (!name)
 		goto end;
 	/* get the name */
-	bp = name->v;
+	bp = (unsigned char *) name->v;
 	len = i2d_X509_NAME(x509->cert_info->subject, &bp);
 
 	error = 0;
@@ -672,7 +672,7 @@ eay_get_x509subjectaltname(cert, altname, type, pos)
 		if (!*altname)
 			goto end;
 		
-		strlcpy(*altname, gen->d.ia5->data, len);
+		strlcpy(*altname, (char *) gen->d.ia5->data, len);
 		*type = gen->type;
 		error = 0;
 	}
@@ -789,7 +789,7 @@ mem2x509(cert)
     {
 	u_char *bp;
 
-	bp = cert->v;
+	bp = (unsigned char *) cert->v;
 
 	x509 = d2i_X509(NULL, &bp, cert->l);
     }
@@ -847,7 +847,7 @@ eay_get_x509cert(path)
 		X509_free(x509);
 		return NULL;
 	}
-	bp = cert->v;
+	bp = (unsigned char *) cert->v;
 	error = i2d_X509(x509, &bp);
 	X509_free(x509);
 
@@ -877,7 +877,7 @@ eay_check_x509sign(source, sig, cert)
 	EVP_PKEY *evp;
 	int res;
 
-	bp = cert->v;
+	bp = (unsigned char *) cert->v;
 
 	x509 = d2i_X509(NULL, &bp, cert->l);
 	if (x509 == NULL) {
@@ -944,7 +944,7 @@ eay_get_pkcs1privkey(path)
 	pkey = vmalloc(pkeylen);
 	if (pkey == NULL)
 		goto end;
-	bp = pkey->v;
+	bp = (unsigned char *) pkey->v;
 	pkeylen = i2d_PrivateKey(evp, &bp);
 	if (pkeylen == 0)
 		goto end;
@@ -1000,7 +1000,7 @@ eay_get_pkcs1pubkey(path)
 	pkey = vmalloc(pkeylen);
 	if (pkey == NULL)
 		goto end;
-	bp = pkey->v;
+	bp = (unsigned char *) pkey->v;
 	pkeylen = i2d_PublicKey(evp, &bp);
 	if (pkeylen == 0)
 		goto end;
@@ -1022,7 +1022,7 @@ eay_get_x509sign(src, privkey)
 	vchar_t *src, *privkey;
 {
 	EVP_PKEY *evp;
-	u_char *bp = privkey->v;
+	u_char *bp = (unsigned char *) privkey->v;
 	vchar_t *sig = NULL;
 	int len;
 	int pad = RSA_PKCS1_PADDING;
@@ -1060,7 +1060,8 @@ eay_rsa_sign(vchar_t *src, RSA *rsa)
 	if (sig == NULL)
 		return NULL;
 
-	len = RSA_private_encrypt(src->l, src->v, sig->v, rsa, pad);
+	len = RSA_private_encrypt(src->l, (unsigned char *) src->v, 
+			(unsigned char *) sig->v, rsa, pad);
 
 	if (len == 0 || len != sig->l) {
 		vfree(sig);
@@ -1087,7 +1088,8 @@ eay_rsa_verify(src, sig, rsa)
 		return -1;
 	}
 
-	len = RSA_public_decrypt(sig->l, sig->v, xbuf->v, rsa, pad);
+	len = RSA_public_decrypt(sig->l, (unsigned char *) sig->v, 
+			(unsigned char *) xbuf->v, rsa, pad);
 	if (len == 0 || len != src->l) {
 		plog(LLV_ERROR, LOCATION, NULL, "%s\n", eay_strerror());
 		vfree(xbuf);
@@ -1151,13 +1153,15 @@ evp_crypt(vchar_t *data, vchar_t *key, vchar_t *iv, const EVP_CIPHER *e, int enc
 
 	EVP_CIPHER_CTX_init(&ctx);
 
-	if (!EVP_CipherInit(&ctx, e, key->v, iv->v, enc)) {
+	if (!EVP_CipherInit(&ctx, e, (unsigned char *) key->v, 
+				(unsigned char *) iv->v, enc)) {
 		OpenSSL_BUG();
 		vfree(res);
 		return NULL;
 	}
 	
-	if (!EVP_Cipher(&ctx, res->v, data->v, data->l)) {
+	if (!EVP_Cipher(&ctx, (unsigned char *) res->v, 
+				(unsigned char *) data->v, data->l)) {
 		OpenSSL_BUG();
 		vfree(res);
 		return NULL;
@@ -1179,10 +1183,13 @@ evp_keylen(int len, const EVP_CIPHER *e)
 {
 	if (!e)
 		return -1;
-	if (len != 0 && len != EVP_CIPHER_key_length(e))
+	/* EVP functions return lengths in bytes, ipsec-tools
+	 * uses lengths in bits, therefore conversion is required. --AK
+	 */
+	if (len != 0 && len != (EVP_CIPHER_key_length(e) << 3))
 		return -1;
 	
-	return EVP_CIPHER_key_length(e);
+	return EVP_CIPHER_key_length(e) << 3;
 }
 
 /*
@@ -1523,7 +1530,7 @@ eay_hmacsha2_512_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	HMAC_Update((HMAC_CTX *)c, data->v, data->l);
+	HMAC_Update((HMAC_CTX *)c, (unsigned char *) data->v, data->l);
 }
 
 vchar_t *
@@ -1536,7 +1543,7 @@ eay_hmacsha2_512_final(c)
 	if ((res = vmalloc(SHA512_DIGEST_LENGTH)) == 0)
 		return NULL;
 
-	HMAC_Final((HMAC_CTX *)c, res->v, &l);
+	HMAC_Final((HMAC_CTX *)c, (unsigned char *) res->v, &l);
 	res->l = l;
 	HMAC_cleanup((HMAC_CTX *)c);
 	(void)racoon_free(c);
@@ -1580,7 +1587,7 @@ eay_hmacsha2_384_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	HMAC_Update((HMAC_CTX *)c, data->v, data->l);
+	HMAC_Update((HMAC_CTX *)c, (unsigned char *) data->v, data->l);
 }
 
 vchar_t *
@@ -1593,7 +1600,7 @@ eay_hmacsha2_384_final(c)
 	if ((res = vmalloc(SHA384_DIGEST_LENGTH)) == 0)
 		return NULL;
 
-	HMAC_Final((HMAC_CTX *)c, res->v, &l);
+	HMAC_Final((HMAC_CTX *)c, (unsigned char *) res->v, &l);
 	res->l = l;
 	HMAC_cleanup((HMAC_CTX *)c);
 	(void)racoon_free(c);
@@ -1637,7 +1644,7 @@ eay_hmacsha2_256_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	HMAC_Update((HMAC_CTX *)c, data->v, data->l);
+	HMAC_Update((HMAC_CTX *)c, (unsigned char *) data->v, data->l);
 }
 
 vchar_t *
@@ -1650,7 +1657,7 @@ eay_hmacsha2_256_final(c)
 	if ((res = vmalloc(SHA256_DIGEST_LENGTH)) == 0)
 		return NULL;
 
-	HMAC_Final((HMAC_CTX *)c, res->v, &l);
+	HMAC_Final((HMAC_CTX *)c, (unsigned char *) res->v, &l);
 	res->l = l;
 	HMAC_cleanup((HMAC_CTX *)c);
 	(void)racoon_free(c);
@@ -1695,7 +1702,7 @@ eay_hmacsha1_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	HMAC_Update((HMAC_CTX *)c, data->v, data->l);
+	HMAC_Update((HMAC_CTX *)c, (unsigned char *) data->v, data->l);
 }
 
 vchar_t *
@@ -1708,7 +1715,7 @@ eay_hmacsha1_final(c)
 	if ((res = vmalloc(SHA_DIGEST_LENGTH)) == 0)
 		return NULL;
 
-	HMAC_Final((HMAC_CTX *)c, res->v, &l);
+	HMAC_Final((HMAC_CTX *)c, (unsigned char *) res->v, &l);
 	res->l = l;
 	HMAC_cleanup((HMAC_CTX *)c);
 	(void)racoon_free(c);
@@ -1752,7 +1759,7 @@ eay_hmacmd5_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	HMAC_Update((HMAC_CTX *)c, data->v, data->l);
+	HMAC_Update((HMAC_CTX *)c, (unsigned char *) data->v, data->l);
 }
 
 vchar_t *
@@ -1765,7 +1772,7 @@ eay_hmacmd5_final(c)
 	if ((res = vmalloc(MD5_DIGEST_LENGTH)) == 0)
 		return NULL;
 
-	HMAC_Final((HMAC_CTX *)c, res->v, &l);
+	HMAC_Final((HMAC_CTX *)c, (unsigned char *) res->v, &l);
 	res->l = l;
 	HMAC_cleanup((HMAC_CTX *)c);
 	(void)racoon_free(c);
@@ -1799,7 +1806,7 @@ eay_sha2_512_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	SHA512_Update((SHA512_CTX *)c, data->v, data->l);
+	SHA512_Update((SHA512_CTX *)c, (unsigned char *) data->v, data->l);
 
 	return;
 }
@@ -1813,7 +1820,7 @@ eay_sha2_512_final(c)
 	if ((res = vmalloc(SHA512_DIGEST_LENGTH)) == 0)
 		return(0);
 
-	SHA512_Final(res->v, (SHA512_CTX *)c);
+	SHA512_Final((unsigned char *) res->v, (SHA512_CTX *)c);
 	(void)racoon_free(c);
 
 	return(res);
@@ -1859,7 +1866,7 @@ eay_sha2_384_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	SHA384_Update((SHA384_CTX *)c, data->v, data->l);
+	SHA384_Update((SHA384_CTX *)c, (unsigned char *) data->v, data->l);
 
 	return;
 }
@@ -1873,7 +1880,7 @@ eay_sha2_384_final(c)
 	if ((res = vmalloc(SHA384_DIGEST_LENGTH)) == 0)
 		return(0);
 
-	SHA384_Final(res->v, (SHA384_CTX *)c);
+	SHA384_Final((unsigned char *) res->v, (SHA384_CTX *)c);
 	(void)racoon_free(c);
 
 	return(res);
@@ -1919,7 +1926,7 @@ eay_sha2_256_update(c, data)
 	caddr_t c;
 	vchar_t *data;
 {
-	SHA256_Update((SHA256_CTX *)c, data->v, data->l);
+	SHA256_Update((SHA256_CTX *)c, (unsigned char *) data->v, data->l);
 
 	return;
 }
@@ -1933,7 +1940,7 @@ eay_sha2_256_final(c)
 	if ((res = vmalloc(SHA256_DIGEST_LENGTH)) == 0)
 		return(0);
 
-	SHA256_Final(res->v, (SHA256_CTX *)c);
+	SHA256_Final((unsigned char *) res->v, (SHA256_CTX *)c);
 	(void)racoon_free(c);
 
 	return(res);
@@ -1992,7 +1999,7 @@ eay_sha1_final(c)
 	if ((res = vmalloc(SHA_DIGEST_LENGTH)) == 0)
 		return(0);
 
-	SHA1_Final(res->v, (SHA_CTX *)c);
+	SHA1_Final((unsigned char *) res->v, (SHA_CTX *)c);
 	(void)racoon_free(c);
 
 	return(res);
@@ -2050,7 +2057,7 @@ eay_md5_final(c)
 	if ((res = vmalloc(MD5_DIGEST_LENGTH)) == 0)
 		return(0);
 
-	MD5_Final(res->v, (MD5_CTX *)c);
+	MD5_Final((unsigned char *) res->v, (MD5_CTX *)c);
 	(void)racoon_free(c);
 
 	return(res);
@@ -2157,7 +2164,7 @@ eay_dh_compute(prime, g, pub, priv, pub2, key)
 	BIGNUM *dh_pub = NULL;
 	DH *dh = NULL;
 	int l;
-	caddr_t v = NULL;
+	unsigned char *v = NULL;
 	int error = -1;
 
 	/* make public number to compute */
@@ -2181,7 +2188,7 @@ eay_dh_compute(prime, g, pub, priv, pub2, key)
 	if (!BN_set_word(dh->g, g))
 		goto end;
 
-	if ((v = (caddr_t)racoon_calloc(prime->l, sizeof(u_char))) == NULL)
+	if ((v = racoon_calloc(prime->l, sizeof(u_char))) == NULL)
 		goto end;
 	if ((l = DH_compute_key(v, dh_pub, dh)) == -1)
 		goto end;
@@ -2217,7 +2224,7 @@ eay_v2bn(bn, var)
 	BIGNUM **bn;
 	vchar_t *var;
 {
-	if ((*bn = BN_bin2bn(var->v, var->l, NULL)) == NULL)
+	if ((*bn = BN_bin2bn((unsigned char *) var->v, var->l, NULL)) == NULL)
 		return -1;
 
 	return 0;
@@ -2232,7 +2239,7 @@ eay_bn2v(var, bn)
 	if (*var == NULL)
 		return(-1);
 
-	(*var)->l = BN_bn2bin(bn, (*var)->v);
+	(*var)->l = BN_bn2bin(bn, (unsigned char *) (*var)->v);
 
 	return 0;
 }
@@ -2322,8 +2329,9 @@ binbuf_pubkey2rsa(vchar_t *binbuf)
 		goto out;
 	}
 
-	exp = BN_bin2bn(binbuf->v + 1, binbuf->v[0], NULL);
-	mod = BN_bin2bn(binbuf->v + binbuf->v[0] + 1, binbuf->l - binbuf->v[0] - 1, NULL);
+	exp = BN_bin2bn((unsigned char *) (binbuf->v + 1), binbuf->v[0], NULL);
+	mod = BN_bin2bn((unsigned char *) (binbuf->v + binbuf->v[0] + 1), 
+			binbuf->l - binbuf->v[0] - 1, NULL);
 	rsa_pub = RSA_new();
 
 	if (!exp || !mod || !rsa_pub) {
@@ -2389,7 +2397,7 @@ bignum_pubkey2rsa(BIGNUM *in)
 		return NULL;
 	}
 	
-	BN_bn2bin(in, binbuf->v);
+	BN_bn2bin(in, (unsigned char *) binbuf->v);
 
 	rsa_pub = binbuf_pubkey2rsa(binbuf);
 
