@@ -1,4 +1,4 @@
-/*	$KAME: eaytest.c,v 1.42 2003/11/13 19:51:43 sakane Exp $	*/
+/* $Id: eaytest.c,v 1.20 2004/11/18 15:14:44 ludvigm Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -59,14 +59,13 @@
 #include "oakley.h"
 #include "dhgroup.h"
 #include "crypto_openssl.h"
+#include "gnuc.h"
 
 #include "../../package_version.h"
 
 #define PVDUMP(var) hexdump((var)->v, (var)->l)
 
-#define CERTTEST_BROKEN
-
-int f_foreground = 1;
+/*#define CERTTEST_BROKEN */
 
 /* prototype */
 
@@ -102,7 +101,7 @@ rsa_verify_with_pubkey(src, sig, pubkey_txt)
 		printf ("PEM_read_PUBKEY(): %s\n", eay_strerror());
 		return -1;
 	}
-	error = eay_rsa_verify(src, sig, evp);
+	error = eay_check_rsasign(src, sig, evp->pkey.rsa);
 
 	return error;
 }
@@ -159,7 +158,7 @@ rsatest(ac, av)
 	src.l = strlen(text);
 
 	/* sign */
-	sig = eay_rsa_sign(&src, priv);
+	sig = eay_get_x509sign(&src, priv);
 	if (sig == NULL) {
 		printf("sign failed. %s\n", eay_strerror());
 		return -1;
@@ -228,8 +227,11 @@ certtest(ac, av)
 	printf("\n**Test for Certificate.**\n");
 
     {
-	char dnstr[] = "C=JP, ST=Kanagawa, L=Fujisawa, O=WIDE Project, OU=KAME Project, CN=Shoichi Sakane/Email=sakane@kame.net";
 	vchar_t *asn1dn = NULL, asn1dn0;
+#ifdef ORIG_DN
+	char dnstr[] = "C=JP, ST=Kanagawa, L=Fujisawa, O=WIDE Project, OU=KAME Project, CN=Shoichi Sakane/Email=sakane@kame.net";
+	char *dnstr_w1 = 0;
+	char *dnstr_w2 = 0;
 	char dn0[] = {
 		0x30,0x81,0x9a,0x31,0x0b,0x30,0x09,0x06,
 		0x03,0x55,0x04,0x06,0x13,0x02,0x4a,0x50,
@@ -254,6 +256,29 @@ certtest(ac, av)
 		0x6b,0x61,0x6e,0x65,0x40,0x6b,0x61,0x6d,
 		0x65,0x2e,0x6e,0x65,0x74,
 	};
+#else /* not ORIG_DN */
+	char dnstr[] = "C=JP, ST=Kanagawa, L=Fujisawa, O=WIDE Project, OU=KAME Project, CN=Shoichi Sakane";
+	char dnstr_w1[] = "C=JP, ST=Kanagawa, L=Fujisawa, O=WIDE Project, OU=*, CN=Shoichi Sakane";
+	char dnstr_w2[] = "C=JP, ST=Kanagawa, L=Fujisawa, O=WIDE Project, OU=KAME Project, CN=*";
+	char dn0[] = {
+		0x30,0x7a,0x31,0x0b,0x30,0x09,0x06,0x03,
+		0x55,0x04,0x06,0x13,0x02,0x4a,0x50,0x31,
+		0x11,0x30,0x0f,0x06,0x03,0x55,0x04,0x08,
+		0x13,0x08,0x4b,0x61,0x6e,0x61,0x67,0x61,
+		0x77,0x61,0x31,0x11,0x30,0x0f,0x06,0x03,
+		0x55,0x04,0x07,0x13,0x08,0x46,0x75,0x6a,
+		0x69,0x73,0x61,0x77,0x61,0x31,0x15,0x30,
+		0x13,0x06,0x03,0x55,0x04,0x0a,0x13,0x0c,
+		0x57,0x49,0x44,0x45,0x20,0x50,0x72,0x6f,
+		0x6a,0x65,0x63,0x74,0x31,0x15,0x30,0x13,
+		0x06,0x03,0x55,0x04,0x0b,0x13,0x0c,0x4b,
+		0x41,0x4d,0x45,0x20,0x50,0x72,0x6f,0x6a,
+		0x65,0x63,0x74,0x31,0x17,0x30,0x15,0x06,
+		0x03,0x55,0x04,0x03,0x13,0x0e,0x53,0x68,
+		0x6f,0x69,0x63,0x68,0x69,0x20,0x53,0x61,
+		0x6b,0x61,0x6e,0x65,
+	};
+#endif /* ORIG_DN */
 
 	printf("check to convert the string into subjectName.\n");
 	printf("%s\n", dnstr);
@@ -263,7 +288,17 @@ certtest(ac, av)
 
 	asn1dn = eay_str2asn1dn(dnstr, strlen(dnstr));
 	if (asn1dn == NULL || asn1dn->l != asn1dn0.l)
-		errx(1, "asn1dn length mismatched.\n");
+#ifdef OUTPUT_VALID_ASN1DN
+	{
+		unsigned char *cp; int  i;
+		printf("asn1dn length mismatched (%zu != %zu).\n", asn1dn ? asn1dn->l : -1, asn1dn0.l);
+		for (cp = asn1dn->v, i = 0; i < asn1dn->l; i++)
+		    printf ("0x%02x,", *cp++);
+		exit (1);
+	}
+#else
+		errx(1, "asn1dn length mismatched (%zu != %zu).\n", asn1dn ? asn1dn->l : -1, asn1dn0.l);
+#endif
 
 	/*
 	 * NOTE: The value pointed by "<==" above is different from the
@@ -274,21 +309,50 @@ certtest(ac, av)
 		errx(1, "asn1dn mismatched.\n");
 	vfree(asn1dn);
 
-	printf("succeed.\n");
+	printf("exact match: succeed.\n");
+
+	if (dnstr_w1) {
+		asn1dn = eay_str2asn1dn(dnstr_w1, strlen(dnstr_w1));
+		if (asn1dn == NULL || asn1dn->l == asn1dn0.l)
+			errx(1, "asn1dn length wrong for wildcard 1\n");
+		if (eay_cmp_asn1dn(&asn1dn0,  asn1dn))
+			errx(1, "asn1dn mismatched for wildcard 1.\n");
+		vfree(asn1dn);
+		printf("wildcard 1 match: succeed.\n");
+	}
+
+	if (dnstr_w1) {
+		asn1dn = eay_str2asn1dn(dnstr_w2, strlen(dnstr_w2));
+		if (asn1dn == NULL || asn1dn->l == asn1dn0.l)
+			errx(1, "asn1dn length wrong for wildcard 2\n");
+		if (eay_cmp_asn1dn(&asn1dn0,  asn1dn))
+			errx(1, "asn1dn mismatched for wildcard 2.\n");
+		vfree(asn1dn);
+		printf("wildcard 2 match: succeed.\n");
+	}
+
     }
-	eay_init_error();
+	eay_init();
 
 	/* get certs */
 	if (ac > 1) {
 		certpath = *(av + 1);
 		certs = getcerts(certpath);
 	} else {
+#ifdef ORIG_DN
 		printf("\nCAUTION: These certificates are probably invalid "
 			"on your environment because you don't have their "
 			"issuer's certs in your environment.\n\n");
 
 		certpath = "/usr/local/openssl/certs";
 		certs = getcerts(NULL);
+#else
+		printf("\nWARNING: The main certificates are probably invalid "
+			"on your environment\nbecause you don't have their "
+			"issuer's certs in your environment\nso not doing "
+			"this test.\n\n");
+		return (0);
+#endif
 	}
 
 	while (*certs != NULL) {
@@ -310,7 +374,7 @@ certtest(ac, av)
 		/* print ASN.1 of subject name */
 		vstr = eay_get_x509asn1subjectname(&c);
 		if (!vstr)
-			return;
+			return 0;
 		PVDUMP(vstr);
 		printf("\n");
 		vfree(vstr);
@@ -331,13 +395,15 @@ certtest(ac, av)
 		}
 	    }
 
-		error = eay_check_x509cert(&c, certpath, 1);
+		/* NULL => name of the certificate file */
+		error = eay_check_x509cert(&c, certpath, NULL, 1);
 		if (error)
 			printf("ERROR: cert is invalid.\n");
 		printf("\n");
 
 		certs++;
 	}
+	return 0;
 }
 
 static char **
@@ -942,6 +1008,7 @@ main(ac, av)
 	int i;
 	int len = sizeof(func)/sizeof(func[0]);
 
+	f_foreground = 1;
 	ploginit();
 
 	printf ("\nTestsuite of the %s\nlinked with %s\n\n", TOP_PACKAGE_STRING, eay_version());
@@ -984,3 +1051,4 @@ Usage()
 #endif
 	exit(1);
 }
+
