@@ -1,4 +1,4 @@
-/* $Id: grabmyaddr.c,v 1.23 2004/11/20 15:53:27 monas Exp $ */
+/* $Id: grabmyaddr.c,v 1.23.2.1 2005/04/21 08:57:16 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -840,7 +840,7 @@ initmyaddr()
 #ifdef __linux__
    {
 	struct sockaddr_nl nl;
-	int addr_len;
+	u_int addr_len;
 
 	memset(&nl, 0, sizeof(nl));
 	nl.nl_family = AF_NETLINK;
