@@ -138,7 +138,7 @@ gssapi_get_default_name(struct ph1handle *iph1, int remote, gss_name_t *service)
 
 	sa = remote ? iph1->remote : iph1->local;
 
-	if (getnameinfo(sa, sa->sa_len, name, NI_MAXHOST, NULL, 0, 0) != 0)
+	if (getnameinfo(sa, sysdep_sa_len(sa), name, NI_MAXHOST, NULL, 0, 0) != 0)
 		return -1;
 
 	name_token.length = asprintf((char **)&name_token.value,
