@@ -1,4 +1,4 @@
-/*	$KAME: handler.c,v 1.57 2002/01/21 08:45:54 sakane Exp $	*/
+/* $Id: handler.c,v 1.13 2004/11/21 19:36:26 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -55,7 +55,12 @@
 #include "policy.h"
 #include "proposal.h"
 #include "isakmp_var.h"
+#include "evt.h"
 #include "isakmp.h"
+#ifdef ENABLE_HYBRID
+#include "isakmp_xauth.h"  
+#include "isakmp_cfg.h"
+#endif
 #include "isakmp_inf.h"
 #include "oakley.h"
 #include "remoteconf.h"
@@ -123,7 +128,7 @@ getph1byindex0(index)
 }
 
 /*
- * search for isakmpsa handler by remote address.
+ * search for isakmpsa handler by source and remote address.
  * don't use port number to search because this function search
  * with phase 2's destinaion.
  */
@@ -144,6 +149,27 @@ getph1byaddr(local, remote)
 	return NULL;
 }
 
+/*
+ * search for isakmpsa handler by remote address.
+ * don't use port number to search because this function search
+ * with phase 2's destinaion.
+ */
+struct ph1handle *
+getph1bydstaddr(remote)
+	struct sockaddr *remote;
+{
+	struct ph1handle *p;
+
+	LIST_FOREACH(p, &ph1tree, chain) {
+		if (p->status == PHASE1ST_EXPIRED)
+			continue;
+		if (cmpsaddrwop(remote, p->remote) == 0)
+			return p;
+	}
+
+	return NULL;
+}
+
 /*
  * dump isakmp-sa
  */
@@ -198,6 +224,14 @@ newph1()
 
 	iph1->status = PHASE1ST_SPAWN;
 
+#ifdef ENABLE_DPD
+	iph1->dpd_support = 0;
+	iph1->dpd_lastack = 0;
+	iph1->dpd_seq = 0;
+	iph1->dpd_fails = 0;
+	iph1->dpd_r_u = NULL;
+#endif
+
 	return iph1;
 }
 
@@ -208,6 +242,12 @@ void
 delph1(iph1)
 	struct ph1handle *iph1;
 {
+	/* SA down shell script hook */
+	if (iph1 != NULL)
+		script_hook(iph1, SCRIPT_PHASE1_DOWN);
+
+	EVT_PUSH(iph1->local, iph1->remote, EVTT_PHASE1_DOWN, NULL);
+
 #ifdef ENABLE_NATT
 	if (iph1->natt_flags & NAT_KA_QUEUED)
 		natt_keepalive_remove (iph1->local, iph1->remote);
@@ -227,6 +267,11 @@ delph1(iph1)
 		iph1->local = NULL;
 	}
 
+#ifdef ENABLE_HYBRID
+	if (iph1->mode_cfg)
+		isakmp_cfg_rmstate(iph1);
+#endif
+
 	VPTRINIT(iph1->authstr);
 
 	sched_scrub_param(iph1);
@@ -880,3 +925,29 @@ init_recvdpkt()
 
 	sched_new(lt, sweep_recvdpkt, NULL);
 }
+
+#ifdef ENABLE_HYBRID
+/* 
+ * Retruns 0 if the address was obtained by ISAKMP mode config, 1 otherwise
+ * This should be in isakmp_cfg.c but ph1tree being private, it must be there
+ */
+int
+exclude_cfg_addr(addr)
+	const struct sockaddr *addr;
+{
+	struct ph1handle *p;
+	struct sockaddr_in *sin;
+
+	LIST_FOREACH(p, &ph1tree, chain) {
+		if ((p->mode_cfg != NULL) &&
+		    (p->mode_cfg->flags & ISAKMP_CFG_GOT_ADDR4) &&
+		    (addr->sa_family == AF_INET)) {
+			sin = (struct sockaddr_in *)addr;
+			if (sin->sin_addr.s_addr == p->mode_cfg->addr4.s_addr)
+				return 0;
+		}
+	}
+
+	return 1;
+}
+#endif
