@@ -1,4 +1,4 @@
-/* $Id: ipsec_doi.c,v 1.24.2.2 2005/02/17 13:18:31 vanhu Exp $ */
+/* $Id: ipsec_doi.c,v 1.24.2.5 2005/04/25 22:06:32 manubsd Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -655,7 +655,7 @@ t2isakmpsa(trns, sa)
 			case IPSECDOI_ATTR_SA_LD_TYPE_KB:
 				sa->lifebyte = ipsecdoi_set_ld(val);
 				vfree(val);
-				if (sa->lifetime == 0) {
+				if (sa->lifebyte == 0) {
 					plog(LLV_ERROR, LOCATION, NULL,
 						"invalid life duration.\n");
 					goto err;
@@ -2547,17 +2547,19 @@ setph1attr(sa, buf)
 	int attrlen = 0;
 
 	if (sa->lifetime) {
+		u_int32_t lifetime = htonl((u_int32_t)sa->lifetime);
+
 		attrlen += sizeof(struct isakmp_data)
 			+ sizeof(struct isakmp_data);
 		if (sa->lifetime > 0xffff)
-			attrlen += sizeof(sa->lifetime);
+			attrlen += sizeof(lifetime);
 		if (buf) {
 			p = isakmp_set_attr_l(p, OAKLEY_ATTR_SA_LD_TYPE,
 						OAKLEY_ATTR_SA_LD_TYPE_SEC);
 			if (sa->lifetime > 0xffff) {
-				u_int32_t v = htonl((u_int32_t)sa->lifetime);
 				p = isakmp_set_attr_v(p, OAKLEY_ATTR_SA_LD,
-						(caddr_t)&v, sizeof(v));
+						(caddr_t)&lifetime, 
+						sizeof(lifetime));
 			} else {
 				p = isakmp_set_attr_l(p, OAKLEY_ATTR_SA_LD,
 							sa->lifetime);
@@ -2566,17 +2568,19 @@ setph1attr(sa, buf)
 	}
 
 	if (sa->lifebyte) {
+		u_int32_t lifebyte = htonl((u_int32_t)sa->lifebyte);
+		
 		attrlen += sizeof(struct isakmp_data)
 			+ sizeof(struct isakmp_data);
 		if (sa->lifebyte > 0xffff)
-			attrlen += sizeof(sa->lifebyte);
+			attrlen += sizeof(lifebyte);
 		if (buf) {
 			p = isakmp_set_attr_l(p, OAKLEY_ATTR_SA_LD_TYPE,
 						OAKLEY_ATTR_SA_LD_TYPE_KB);
 			if (sa->lifebyte > 0xffff) {
-				u_int32_t v = htonl((u_int32_t)sa->lifebyte);
 				p = isakmp_set_attr_v(p, OAKLEY_ATTR_SA_LD,
-							(caddr_t)&v, sizeof(v));
+							(caddr_t)&lifebyte,
+							sizeof(lifebyte));
 			} else {
 				p = isakmp_set_attr_l(p, OAKLEY_ATTR_SA_LD,
 							sa->lifebyte);
@@ -3427,7 +3431,7 @@ set_identifier(vpp, type, value)
 		if (loglevel >= LLV_DEBUG) {
 			X509_NAME *xn;
 			BIO *bio;
-			unsigned char *ptr = new->v, *buf;
+			unsigned char *ptr = (unsigned char *) new->v, *buf;
 			size_t len;
 			char save;
 
@@ -3581,7 +3585,8 @@ ipsecdoi_sockaddr2id(saddr, prefixlen, ul_proto)
 
 	/* set prefix */
 	if (len2) {
-		u_char *p = new->v + sizeof(struct ipsecdoi_id_b) + len1;
+		u_char *p = (unsigned char *) new->v + 
+			sizeof(struct ipsecdoi_id_b) + len1;
 		u_int bits = prefixlen;
 
 		while (bits >= 8) {
@@ -3691,7 +3696,7 @@ ipsecdoi_id2sockaddr(buf, saddr, prefixlen, ul_proto)
 		plen = 0;
 		max = alen <<3;
 
-		p = buf->v
+		p = (unsigned char *) buf->v
 			+ sizeof(struct ipsecdoi_id_b)
 			+ alen;
 
