@@ -1,4 +1,4 @@
-/* $Id: isakmp.c,v 1.32.2.1 2005/01/24 08:50:03 vanhu Exp $ */
+/* $Id: isakmp.c,v 1.32.2.4 2005/04/21 08:57:17 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -186,8 +186,8 @@ isakmp_handler(so_isakmp)
 	} x;
 	struct sockaddr_storage remote;
 	struct sockaddr_storage local;
-	int remote_len = sizeof(remote);
-	int local_len = sizeof(local);
+	unsigned int remote_len = sizeof(remote);
+	unsigned int local_len = sizeof(local);
 	int len = 0, extralen = 0;
 	u_short port;
 	vchar_t *buf = NULL, *tmpbuf = NULL;
@@ -1346,7 +1346,7 @@ isakmp_parsewoh(np0, gen, len)
 
 		p->type = np;
 		p->len = ntohs(gen->len);
-		if (p->len == 0 || p->len > tlen) {
+		if (p->len < sizeof(struct isakmp_gen) || p->len > tlen) {
 			plog(LLV_DEBUG, LOCATION, NULL,
 				"invalid length of payload\n");
 			vfree(result);
@@ -1821,18 +1821,19 @@ isakmp_ph1expire(iph1)
 {
 	char *src, *dst;
 
-	src = strdup(saddr2str(iph1->local));
-	dst = strdup(saddr2str(iph1->remote));
-	plog(LLV_INFO, LOCATION, NULL,
-		"ISAKMP-SA expired %s-%s spi:%s\n",
-		src, dst,
-		isakmp_pindex(&iph1->index, 0));
-	racoon_free(src);
-	racoon_free(dst);
-
 	SCHED_KILL(iph1->sce);
 
-	iph1->status = PHASE1ST_EXPIRED;
+	if(iph1->status != PHASE1ST_EXPIRED){
+		src = strdup(saddr2str(iph1->local));
+		dst = strdup(saddr2str(iph1->remote));
+		plog(LLV_INFO, LOCATION, NULL,
+			 "ISAKMP-SA expired %s-%s spi:%s\n",
+			 src, dst,
+			 isakmp_pindex(&iph1->index, 0));
+		racoon_free(src);
+		racoon_free(dst);
+		iph1->status = PHASE1ST_EXPIRED;
+	}
 
 	/*
 	 * the phase1 deletion is postponed until there is no phase2.
