@@ -1,4 +1,4 @@
-/*	$KAME: isakmp_agg.c,v 1.56 2003/11/13 02:30:20 sakane Exp $	*/
+/* $Id: isakmp_agg.c,v 1.19.2.3 2005/01/26 19:39:26 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -63,6 +63,7 @@
 #include "remoteconf.h"
 #include "isakmp_var.h"
 #include "isakmp.h"
+#include "evt.h"
 #include "oakley.h"
 #include "handler.h"
 #include "ipsec_doi.h"
@@ -70,6 +71,13 @@
 #include "pfkey.h"
 #include "isakmp_agg.h"
 #include "isakmp_inf.h"
+#ifdef ENABLE_HYBRID
+#include "isakmp_xauth.h"
+#include "isakmp_cfg.h"
+#endif
+#ifdef ENABLE_FRAG
+#include "isakmp_frag.h"
+#endif
 #include "vendorid.h"
 #include "strnames.h"
 
@@ -104,11 +112,22 @@ agg_i1send(iph1, msg)
 	int error = -1;
 #ifdef ENABLE_NATT
 	vchar_t *vid_natt[MAX_NATT_VID_COUNT];
-	int i, vid_natt_i = 0;
+	int i;
+#endif
+#ifdef ENABLE_HYBRID
+	vchar_t *vid_xauth = NULL;
+	vchar_t *vid_unity = NULL;
+#endif
+#ifdef ENABLE_FRAG
+	vchar_t *vid_frag = NULL;
 #endif
 #ifdef HAVE_GSSAPI
 	int len;
 #endif
+#ifdef ENABLE_DPD
+	vchar_t *vid_dpd = NULL;
+#endif
+
 
 	/* validity check */
 	if (msg != NULL) {
@@ -152,27 +171,35 @@ agg_i1send(iph1, msg)
 	if (iph1->nonce == NULL)
 		goto end;
 
-#ifdef ENABLE_NATT
-	/* Is NAT-T support allowed in the config file? */
-	if (iph1->rmconf->nat_traversal) {
-		/* Advertise NAT-T capability */
-		memset (vid_natt, 0, sizeof (vid_natt));
-#ifdef VENDORID_NATT_00
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_00)) != NULL)
-			vid_natt_i++;
-#endif
-#ifdef VENDORID_NATT_02
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_02)) != NULL)
-			vid_natt_i++;
-#endif
-#ifdef VENDORID_NATT_RFC
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_RFC)) != NULL)
-			vid_natt_i++;
-#endif
+#ifdef ENABLE_HYBRID
+	/* Do we need Xauth VID? */
+	switch (iph1->rmconf->proposal->authmethod) {
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_RSA_R:
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_DSS_R:
+		if ((vid_xauth = set_vendorid(VENDORID_XAUTH)) == NULL)
+			plog(LLV_ERROR, LOCATION, NULL, 
+			     "Xauth vendor ID generation failed\n");
+		if ((vid_unity = set_vendorid(VENDORID_UNITY)) == NULL)
+			plog(LLV_ERROR, LOCATION, NULL, 
+			     "Unity vendor ID generation failed\n");
+		break;
+	default:
+		break;
 	}
 #endif
 
-#ifdef HAVE_SIGNING_C
+#ifdef ENABLE_FRAG
+	if (iph1->rmconf->ike_frag) {
+		vid_frag = set_vendorid(VENDORID_FRAG);
+		if (vid_frag != NULL)
+			vid_frag = isakmp_frag_addcap(vid_frag,
+			    VENDORID_FRAG_AGG);
+		if (vid_frag == NULL)
+			plog(LLV_ERROR, LOCATION, NULL,
+			    "Frag vendorID construction failed\n");
+	}		
+#endif
+
 	/* create CR if need */
 	if (iph1->rmconf->send_cr
 	 && oakley_needcr(iph1->rmconf->proposal->authmethod)
@@ -185,7 +212,7 @@ agg_i1send(iph1, msg)
 			goto end;
 		}
 	}
-#endif
+
 	plog(LLV_DEBUG, LOCATION, NULL, "authmethod is %s\n",
 		s_oakley_attr_method(iph1->rmconf->proposal->authmethod));
 #ifdef HAVE_GSSAPI
@@ -218,13 +245,34 @@ agg_i1send(iph1, msg)
 	if (need_cr)
 		plist = isakmp_plist_append(plist, cr, ISAKMP_NPTYPE_CR);
 
+#ifdef ENABLE_FRAG
+	if (vid_frag)
+		plist = isakmp_plist_append(plist, vid_frag, ISAKMP_NPTYPE_VID);
+#endif
 #ifdef ENABLE_NATT
-	/* set VID payload for NAT-T */
-	for (i = 0; i < vid_natt_i; i++)
-		plist = isakmp_plist_append(plist, vid_natt[i], ISAKMP_NPTYPE_VID);
+	/* set VID payload for NAT-T if NAT-T support allowed in the config file */
+	if (iph1->rmconf->nat_traversal) 
+		plist = isakmp_plist_append_natt_vids(plist, vid_natt);
+	else
+		vid_natt[0]=NULL;
+#endif
+#ifdef ENABLE_HYBRID
+	if (vid_xauth)
+		plist = isakmp_plist_append(plist, 
+		    vid_xauth, ISAKMP_NPTYPE_VID);
+	if (vid_unity)
+		plist = isakmp_plist_append(plist, 
+		    vid_unity, ISAKMP_NPTYPE_VID);
+#endif
+#ifdef ENABLE_DPD
+	if(iph1->rmconf->dpd){
+		vid_dpd = set_vendorid(VENDORID_DPD);
+		if (vid_dpd != NULL)
+			plist = isakmp_plist_append(plist, vid_dpd, ISAKMP_NPTYPE_VID);
+	}
+#endif
 
 	iph1->sendbuf = isakmp_plist_set_all (&plist, iph1);
-#endif
 
 #ifdef HAVE_PRINT_ISAKMP_C
 	isakmp_printpacket(iph1->sendbuf, iph1->local, iph1->remote, 0);
@@ -244,10 +292,18 @@ end:
 		vfree(cr);
 	if (gsstoken)
 		vfree(gsstoken);
+#ifdef ENABLE_FRAG
+	if (vid_frag)
+		vfree(vid_frag);
+#endif
 #ifdef ENABLE_NATT
-	for (i = 0; i < vid_natt_i; i++)
+	for (i = 0; i < MAX_NATT_VID_COUNT && vid_natt[i] != NULL; i++)
 		vfree(vid_natt[i]);
 #endif
+#ifdef ENABLE_DPD
+	if (vid_dpd != NULL)
+		vfree(vid_dpd);
+#endif
 
 	return error;
 }
@@ -271,6 +327,10 @@ agg_i2recv(iph1, msg)
 	int error = -1;
 	int vid_numeric;
 	int ptype;
+#ifdef ENABLE_HYBRID
+	vchar_t *unity_vid;
+	vchar_t *xauth_vid;
+#endif
 #ifdef HAVE_GSSAPI
 	vchar_t *gsstoken = NULL;
 #endif
@@ -334,7 +394,6 @@ agg_i2recv(iph1, msg)
 		case ISAKMP_NPTYPE_HASH:
 			iph1->pl_hash = (struct isakmp_pl_hash *)pa->ptr;
 			break;
-#ifdef HAVE_SIGNING_C
 		case ISAKMP_NPTYPE_CR:
 			if (oakley_savecr(iph1, pa->ptr) < 0)
 				goto end;
@@ -347,12 +406,34 @@ agg_i2recv(iph1, msg)
 			if (isakmp_p2ph(&iph1->sig_p, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
 		case ISAKMP_NPTYPE_VID:
 			vid_numeric = check_vendorid(pa->ptr);
 #ifdef ENABLE_NATT
-			if (iph1->rmconf->nat_traversal && natt_vendorid(vid_numeric))
-			  natt_handle_vendorid(iph1, vid_numeric);
+			if (iph1->rmconf->nat_traversal && 
+			    natt_vendorid(vid_numeric))
+				natt_handle_vendorid(iph1, vid_numeric);
+#endif
+#ifdef ENABLE_HYBRID
+			switch (vid_numeric) {
+			case VENDORID_XAUTH:
+				iph1->mode_cfg->flags |= 
+				    ISAKMP_CFG_VENDORID_XAUTH;
+				break;
+
+			case VENDORID_UNITY:
+				iph1->mode_cfg->flags |= 
+				    ISAKMP_CFG_VENDORID_UNITY;
+				break;
+			default:
+				break;
+			}
+#endif
+#ifdef ENABLE_DPD
+			if (vid_numeric == VENDORID_DPD && iph1->rmconf->dpd) {
+				iph1->dpd_support=1;
+				plog(LLV_DEBUG, LOCATION, NULL,
+					 "remote supports DPD\n");
+			}
 #endif
 			break;
 		case ISAKMP_NPTYPE_N:
@@ -427,14 +508,15 @@ agg_i2recv(iph1, msg)
 		struct natd_payload *natd = NULL;
 		int natd_verified;
 		
-		plog(LLV_INFO, LOCATION, iph1->remote, "Selected NAT-T version: %s\n",
-			vendorid_strings[iph1->natt_options->version]);
+		plog(LLV_INFO, LOCATION, iph1->remote,
+		     "Selected NAT-T version: %s\n",
+		     vid_string_by_id(iph1->natt_options->version));
 
 		/* set both bits first so that we can clear them
 		   upon verifying hashes */
 		iph1->natt_flags |= NAT_DETECTED;
                         
-		TAILQ_FOREACH (natd, &natd_tree, chain) {
+		while ((natd = TAILQ_FIRST(&natd_tree)) != NULL) {
 			/* this function will clear appropriate bits bits 
 			   from iph1->natt_flags */
 			natd_verified = natt_compare_addr_hash (iph1,
@@ -445,7 +527,8 @@ agg_i2recv(iph1, msg)
 				natd_verified ? "verified" : "doesn't match");
 			
 			vfree (natd->payload);
-			// BUGBUG: Don't free it here. It's reused for next iteration.
+
+			TAILQ_REMOVE(&natd_tree, natd, chain);
 			racoon_free (natd);
 		}
 
@@ -482,16 +565,16 @@ agg_i2recv(iph1, msg)
 			/* message printed inner oakley_validate_auth() */
 			goto end;
 		}
+		EVT_PUSH(iph1->local, iph1->remote, 
+		    EVTT_PEERPH1AUTH_FAILED, NULL);
 		isakmp_info_send_n1(iph1, ptype, NULL);
 		goto end;
 	}
 
-#ifdef HAVE_SIGNING_C
 	if (oakley_checkcr(iph1) < 0) {
 		/* Ignore this error in order to be interoperability. */
 		;
 	}
-#endif
 
 	/* change status of isakmp status entry */
 	iph1->status = PHASE1ST_MSG2RECEIVED;
@@ -561,9 +644,12 @@ agg_i2send(iph1, msg)
 		/* set HASH payload */
 		plist = isakmp_plist_append(plist, iph1->hash, ISAKMP_NPTYPE_HASH);
 		break;
-#ifdef HAVE_SIGNING_C
 	case OAKLEY_ATTR_AUTH_METHOD_DSSSIG:
 	case OAKLEY_ATTR_AUTH_METHOD_RSASIG:
+#ifdef ENABLE_HYBRID
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_RSA_I:
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_DSS_I:
+#endif  
 		/* XXX if there is CR or not ? */
 
 		if (oakley_getmycert(iph1) < 0)
@@ -582,7 +668,7 @@ agg_i2send(iph1, msg)
 		/* add SIG payload */
 		plist = isakmp_plist_append(plist, iph1->sig, ISAKMP_NPTYPE_SIG);
 		break;
-#endif
+
 	case OAKLEY_ATTR_AUTH_METHOD_RSAENC:
 	case OAKLEY_ATTR_AUTH_METHOD_RSAREV:
 		break;
@@ -726,17 +812,48 @@ agg_r1recv(iph1, msg)
 			break;
 		case ISAKMP_NPTYPE_VID:
 			vid_numeric = check_vendorid(pa->ptr);
+
 #ifdef ENABLE_NATT
-			if (iph1->rmconf->nat_traversal && natt_vendorid(vid_numeric))
+			if (iph1->rmconf->nat_traversal &&
+			    natt_vendorid(vid_numeric)) {
 				natt_handle_vendorid(iph1, vid_numeric);
+				break;
+			}
+#endif
+#ifdef ENABLE_HYBRID
+			switch (vid_numeric) {
+			case VENDORID_XAUTH:
+				iph1->mode_cfg->flags |= 
+				    ISAKMP_CFG_VENDORID_XAUTH;
+				break;
+
+			case VENDORID_UNITY:
+				iph1->mode_cfg->flags |= 
+				    ISAKMP_CFG_VENDORID_UNITY;
+				break;
+			default:
+				break;
+			}
+#endif
+#ifdef ENABLE_DPD
+			if (vid_numeric == VENDORID_DPD && iph1->rmconf->dpd) {
+				iph1->dpd_support=1;
+				plog(LLV_DEBUG, LOCATION, NULL,
+					 "remote supports DPD\n");
+			}
+#endif
+#ifdef ENABLE_FRAG
+			if ((vid_numeric == VENDORID_FRAG) &&
+			    (vendorid_frag_cap(pa->ptr) & VENDORID_FRAG_AGG))
+				iph1->frag = 1;
 #endif
 			break;
-#ifdef HAVE_SIGNING_C
+
 		case ISAKMP_NPTYPE_CR:
 			if (oakley_savecr(iph1, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
+
 #ifdef HAVE_GSSAPI
 		case ISAKMP_NPTYPE_GSS:
 			if (isakmp_p2ph(&gsstoken, pa->ptr) < 0)
@@ -766,8 +883,9 @@ agg_r1recv(iph1, msg)
 
 #ifdef ENABLE_NATT
 	if (NATT_AVAILABLE(iph1))
-		plog(LLV_INFO, LOCATION, iph1->remote, "Selected NAT-T version: %s\n",
-			vendorid_strings[iph1->natt_options->version]);
+		plog(LLV_INFO, LOCATION, iph1->remote,
+		     "Selected NAT-T version: %s\n",
+		     vid_string_by_id(iph1->natt_options->version));
 #endif
 
 	/* check SA payload and set approval SA for use */
@@ -778,12 +896,10 @@ agg_r1recv(iph1, msg)
 		goto end;
 	}
 
-#ifdef HAVE_SIGNING_C
 	if (oakley_checkcr(iph1) < 0) {
 		/* Ignore this error in order to be interoperability. */
 		;
 	}
-#endif
 
 	iph1->status = PHASE1ST_MSG1RECEIVED;
 
@@ -823,10 +939,17 @@ agg_r1send(iph1, msg)
 	vchar_t *cr = NULL;
 	vchar_t *vid = NULL;
 	int error = -1;
+#ifdef ENABLE_HYBRID
+	vchar_t *xauth_vid = NULL;
+	vchar_t *unity_vid = NULL;
+#endif
 #ifdef ENABLE_NATT
 	vchar_t *vid_natt = NULL;
 	vchar_t *natd[2] = { NULL, NULL };
 #endif
+#ifdef ENABLE_DPD
+	vchar_t *vid_dpd = NULL;
+#endif
 
 #ifdef HAVE_GSSAPI
 	int gsslen;
@@ -891,7 +1014,6 @@ agg_r1send(iph1, msg)
 		goto end;
 	}
 
-#ifdef HAVE_SIGNING_C
 	/* create CR if need */
 	if (iph1->rmconf->send_cr
 	 && oakley_needcr(iph1->approval->authmethod)
@@ -904,7 +1026,6 @@ agg_r1send(iph1, msg)
 			goto end;
 		}
 	}
-#endif
 
 #ifdef ENABLE_NATT
 	/* Has the peer announced NAT-T? */
@@ -927,6 +1048,11 @@ agg_r1send(iph1, msg)
 		}
 	}
 #endif
+#ifdef ENABLE_DPD
+	/* Only send DPD support if remote announced DPD and if DPD support is active */
+	if (iph1->dpd_support && iph1->rmconf->dpd)
+		vid_dpd = set_vendorid(VENDORID_DPD);
+#endif
 
 	switch (iph1->approval->authmethod) {
 	case OAKLEY_ATTR_AUTH_METHOD_PSKEY:
@@ -949,24 +1075,16 @@ agg_r1send(iph1, msg)
 		if (vid)
 			plist = isakmp_plist_append(plist, vid, ISAKMP_NPTYPE_VID);
 
-#ifdef ENABLE_NATT
-		/* append NAT-T payloads */
-		if (vid_natt) {
-			/* chosen VID */
-			plist = isakmp_plist_append(plist, vid_natt, ISAKMP_NPTYPE_VID);
-			/* NAT-D */
-			plist = isakmp_plist_append(plist, natd[0], iph1->natt_options->payload_nat_d);
-			plist = isakmp_plist_append(plist, natd[1], iph1->natt_options->payload_nat_d);
-		}
-#endif
-
 		/* create isakmp CR payload if needed */
 		if (need_cr)
 			plist = isakmp_plist_append(plist, cr, ISAKMP_NPTYPE_CR);
 		break;
-#ifdef HAVE_SIGNING_C
 	case OAKLEY_ATTR_AUTH_METHOD_DSSSIG:
 	case OAKLEY_ATTR_AUTH_METHOD_RSASIG:
+#ifdef ENABLE_HYBRID
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_RSA_I:
+	case OAKLEY_ATTR_AUTH_METHOD_HYBRID_DSS_I:
+#endif
 		/* XXX if there is CR or not ? */
 
 		if (oakley_getmycert(iph1) < 0)
@@ -1001,22 +1119,33 @@ agg_r1send(iph1, msg)
 		if (vid)
 			plist = isakmp_plist_append(plist, vid, ISAKMP_NPTYPE_VID);
 
-#ifdef ENABLE_NATT
-		/* append NAT-T payloads */
-		if (vid_natt) {
-			/* chosen VID */
-			plist = isakmp_plist_append(plist, vid_natt, ISAKMP_NPTYPE_VID);
-			/* NAT-D */
-			plist = isakmp_plist_append(plist, natd[0], iph1->natt_options->payload_nat_d);
-			plist = isakmp_plist_append(plist, natd[1], iph1->natt_options->payload_nat_d);
+#ifdef ENABLE_HYBRID
+	if (iph1->mode_cfg->flags & ISAKMP_CFG_VENDORID_XAUTH) {
+		if ((xauth_vid = set_vendorid(VENDORID_XAUTH)) == NULL) {
+			plog(LLV_ERROR, LOCATION, NULL,
+			    "Cannot create Xauth vendor ID\n");
+			goto end;
 		}
+		plist = isakmp_plist_append(plist, 
+		    xauth_vid, ISAKMP_NPTYPE_VID);
+	}
+
+	if (iph1->mode_cfg->flags & ISAKMP_CFG_VENDORID_UNITY) {
+		if ((unity_vid = set_vendorid(VENDORID_UNITY)) == NULL) {
+			plog(LLV_ERROR, LOCATION, NULL,
+			    "Cannot create Unity vendor ID\n");
+			goto end;
+		}
+		plist = isakmp_plist_append(plist, 
+		    unity_vid, ISAKMP_NPTYPE_VID);
+	}
 #endif
 
 		/* create isakmp CR payload if needed */
 		if (need_cr)
 			plist = isakmp_plist_append(plist, cr, ISAKMP_NPTYPE_CR);
 		break;
-#endif
+
 	case OAKLEY_ATTR_AUTH_METHOD_RSAENC:
 	case OAKLEY_ATTR_AUTH_METHOD_RSAREV:
 		break;
@@ -1065,21 +1194,25 @@ agg_r1send(iph1, msg)
 		if (vid)
 			plist = isakmp_plist_append(plist, vid, ISAKMP_NPTYPE_VID);
 
-#ifdef ENABLE_NATT
-		/* append NAT-T payloads */
-		if (vid_natt) {
-			/* chosen VID */
-			plist = isakmp_plist_append(plist, vid_natt, ISAKMP_NPTYPE_VID);
-			/* NAT-D */
-			plist = isakmp_plist_append(plist, natd[0], iph1->natt_options->payload_nat_d);
-			plist = isakmp_plist_append(plist, natd[1], iph1->natt_options->payload_nat_d);
-		}
-#endif
-
 		break;
 #endif
 	}
 
+#ifdef ENABLE_NATT
+	/* append NAT-T payloads */
+	if (vid_natt) {
+		/* chosen VID */
+		plist = isakmp_plist_append(plist, vid_natt, ISAKMP_NPTYPE_VID);
+		/* NAT-D */
+		plist = isakmp_plist_append(plist, natd[0], iph1->natt_options->payload_nat_d);
+		plist = isakmp_plist_append(plist, natd[1], iph1->natt_options->payload_nat_d);
+	}
+#endif
+#ifdef ENABLE_DPD
+	if (vid_dpd)
+		plist = isakmp_plist_append(plist, vid_dpd, ISAKMP_NPTYPE_VID);
+#endif
+
 	iph1->sendbuf = isakmp_plist_set_all (&plist, iph1);
 
 #ifdef HAVE_PRINT_ISAKMP_C
@@ -1107,6 +1240,12 @@ end:
 		vfree(cr);
 	if (vid)
 		vfree(vid);
+#ifdef ENABLE_HYBRID
+	if (xauth_vid)
+		vfree(xauth_vid);
+	if (unity_vid)
+		vfree(unity_vid);
+#endif
 #ifdef HAVE_GSSAPI
 	if (gsstoken)
 		vfree(gsstoken);
@@ -1115,6 +1254,10 @@ end:
 	if (gss_sa != iph1->sa_ret)
 		vfree(gss_sa);
 #endif
+#ifdef ENABLE_DPD
+	if (vid_dpd)
+		vfree(vid_dpd);
+#endif
 
 	return error;
 }
@@ -1177,7 +1320,6 @@ agg_r2recv(iph1, msg0)
 		case ISAKMP_NPTYPE_VID:
 			(void)check_vendorid(pa->ptr);
 			break;
-#ifdef HAVE_SIGNING_C
 		case ISAKMP_NPTYPE_CERT:
 			if (oakley_savecert(iph1, pa->ptr) < 0)
 				goto end;
@@ -1186,7 +1328,6 @@ agg_r2recv(iph1, msg0)
 			if (isakmp_p2ph(&iph1->sig_p, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
 		case ISAKMP_NPTYPE_N:
 			isakmp_check_notify(pa->ptr, iph1);
 			break;
@@ -1244,6 +1385,8 @@ agg_r2recv(iph1, msg0)
 			/* message printed inner oakley_validate_auth() */
 			goto end;
 		}
+		EVT_PUSH(iph1->local, iph1->remote, 
+		    EVTT_PEERPH1AUTH_FAILED, NULL);
 		isakmp_info_send_n1(iph1, ptype, NULL);
 		goto end;
 	}
