@@ -1,4 +1,4 @@
-/* $Id: isakmp_cfg.c,v 1.22 2004/11/30 00:46:09 manubsd Exp $ */
+/* $Id: isakmp_cfg.c,v 1.22.2.1 2005/04/04 21:37:58 manubsd Exp $ */
 
 /*
  * Copyright (C) 2004 Emmanuel Dreyfus
@@ -630,7 +630,7 @@ isakmp_cfg_set(iph1, attrpl)
 			attr++;
 		} else {
 			alen = ntohs(attr->lorv);
-			tlen -= alen;
+			tlen -= (sizeof(*attr) + alen);
 			npp = (char *)attr;
 			attr = (struct isakmp_data *)
 			    (npp + sizeof(*attr) + alen);
