@@ -1,4 +1,4 @@
-/*	$KAME: isakmp_ident.c,v 1.65 2003/11/13 19:11:53 sakane Exp $	*/
+/* $Id: isakmp_ident.c,v 1.12.2.3 2005/01/26 19:39:26 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -63,6 +63,7 @@
 #include "remoteconf.h"
 #include "isakmp_var.h"
 #include "isakmp.h"
+#include "evt.h"
 #include "oakley.h"
 #include "handler.h"
 #include "ipsec_doi.h"
@@ -101,9 +102,11 @@ ident_i1send(iph1, msg)
 	int error = -1;
 #ifdef ENABLE_NATT
 	vchar_t *vid_natt[MAX_NATT_VID_COUNT];
-	int i, vid_natt_i = 0;
+	int i;
+#endif
+#ifdef ENABLE_DPD
+	vchar_t *vid_dpd = NULL;
 #endif
-
 	/* validity check */
 	if (msg != NULL) {
 		plog(LLV_ERROR, LOCATION, NULL,
@@ -129,25 +132,18 @@ ident_i1send(iph1, msg)
 	plist = isakmp_plist_append(plist, iph1->sa, ISAKMP_NPTYPE_SA);
 
 #ifdef ENABLE_NATT
-	/* Is NAT-T support allowed in the config file? */
-	if (iph1->rmconf->nat_traversal) {
-		/* Advertise NAT-T capability */
-		memset (vid_natt, 0, sizeof (vid_natt));
-		#ifdef VENDORID_NATT_00
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_00)) != NULL)
-			vid_natt_i++;
-		#endif
-		#ifdef VENDORID_NATT_02
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_02)) != NULL)
-			vid_natt_i++;
-		#endif
-		#ifdef VENDORID_NATT_RFC
-		if ((vid_natt[vid_natt_i] = set_vendorid(VENDORID_NATT_RFC)) != NULL)
-			vid_natt_i++;
-		#endif
-		/* set VID payload for NAT-T */
-		for (i = 0; i < vid_natt_i; i++)
-			plist = isakmp_plist_append(plist, vid_natt[i], ISAKMP_NPTYPE_VID);
+	/* set VID payload for NAT-T if NAT-T support allowed in the config file */
+	if (iph1->rmconf->nat_traversal) 
+		plist = isakmp_plist_append_natt_vids(plist, vid_natt);
+	else
+		vid_natt[0]=NULL;
+#endif
+#ifdef ENABLE_DPD
+	if(iph1->rmconf->dpd){
+		vid_dpd = set_vendorid(VENDORID_DPD);
+		if (vid_dpd != NULL)
+			plist = isakmp_plist_append(plist, vid_dpd,
+										ISAKMP_NPTYPE_VID);
 	}
 #endif
 
@@ -168,9 +164,13 @@ ident_i1send(iph1, msg)
 
 end:
 #ifdef ENABLE_NATT
-	for (i = 0; i < vid_natt_i; i++)
+	for (i = 0; i < MAX_NATT_VID_COUNT && vid_natt[i] != NULL; i++)
 		vfree(vid_natt[i]);
 #endif
+#ifdef ENABLE_DPD
+	if (vid_dpd != NULL)
+		vfree(vid_dpd);
+#endif
 
 	return error;
 }
@@ -237,6 +237,10 @@ ident_i2recv(iph1, msg)
 #ifdef ENABLE_NATT
 			if (iph1->rmconf->nat_traversal && natt_vendorid(vid_numeric))
 			  natt_handle_vendorid(iph1, vid_numeric);
+#endif
+#ifdef ENABLE_DPD
+			if (vid_numeric == VENDORID_DPD && iph1->rmconf->dpd)
+				iph1->dpd_support=1;
 #endif
 			break;
 		default:
@@ -251,8 +255,9 @@ ident_i2recv(iph1, msg)
 
 #ifdef ENABLE_NATT
 	if (NATT_AVAILABLE(iph1))
-		plog(LLV_INFO, LOCATION, iph1->remote, "Selected NAT-T version: %s\n",
-			vendorid_strings[iph1->natt_options->version]);
+		plog(LLV_INFO, LOCATION, iph1->remote,
+		     "Selected NAT-T version: %s\n",
+		     vid_string_by_id(iph1->natt_options->version));
 #endif
 
 	/* check SA payload and set approval SA for use */
@@ -400,12 +405,10 @@ ident_i3recv(iph1, msg)
 		case ISAKMP_NPTYPE_VID:
 			(void)check_vendorid(pa->ptr);
 			break;
-#ifdef HAVE_SIGNING_C
 		case ISAKMP_NPTYPE_CR:
 			if (oakley_savecr(iph1, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
 #ifdef HAVE_GSSAPI
 		case ISAKMP_NPTYPE_GSS:
 			if (isakmp_p2ph(&gsstoken, pa->ptr) < 0)
@@ -472,12 +475,10 @@ ident_i3recv(iph1, msg)
 		goto end;
 	}
 
-#ifdef HAVE_SIGNING_C
 	if (oakley_checkcr(iph1) < 0) {
 		/* Ignore this error in order to be interoperability. */
 		;
 	}
-#endif
 
 	iph1->status = PHASE1ST_MSG3RECEIVED;
 
@@ -651,7 +652,6 @@ ident_i4recv(iph1, msg0)
 		case ISAKMP_NPTYPE_HASH:
 			iph1->pl_hash = (struct isakmp_pl_hash *)pa->ptr;
 			break;
-#ifdef HAVE_SIGNING_C
 		case ISAKMP_NPTYPE_CERT:
 			if (oakley_savecert(iph1, pa->ptr) < 0)
 				goto end;
@@ -660,7 +660,6 @@ ident_i4recv(iph1, msg0)
 			if (isakmp_p2ph(&iph1->sig_p, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
 #ifdef HAVE_GSSAPI
 		case ISAKMP_NPTYPE_GSS:
 			if (isakmp_p2ph(&gsstoken, pa->ptr) < 0)
@@ -703,6 +702,8 @@ ident_i4recv(iph1, msg0)
 				/* msg printed inner oakley_validate_auth() */
 				goto end;
 			}
+			EVT_PUSH(iph1->local, iph1->remote, 
+			    EVTT_PEERPH1AUTH_FAILED, NULL);
 			isakmp_info_send_n1(iph1, type, NULL);
 			goto end;
 		}
@@ -838,6 +839,10 @@ ident_r1recv(iph1, msg)
 #ifdef ENABLE_NATT
 			if (iph1->rmconf->nat_traversal && natt_vendorid(vid_numeric))
 				natt_handle_vendorid(iph1, vid_numeric);
+#endif
+#ifdef ENABLE_DPD
+			if (vid_numeric == VENDORID_DPD && iph1->rmconf->dpd)
+				iph1->dpd_support=1;
 #endif
 			break;
 		default:
@@ -858,8 +863,9 @@ ident_r1recv(iph1, msg)
 
 #ifdef ENABLE_NATT
 	if (NATT_AVAILABLE(iph1))
-		plog(LLV_INFO, LOCATION, iph1->remote, "Selected NAT-T version: %s\n",
-			vendorid_strings[iph1->natt_options->version]);
+		plog(LLV_INFO, LOCATION, iph1->remote,
+		     "Selected NAT-T version: %s\n",
+		     vid_string_by_id(iph1->natt_options->version));
 #endif
 
 	/* check SA payload and set approval SA for use */
@@ -903,6 +909,9 @@ ident_r1send(iph1, msg)
 #ifdef ENABLE_NATT
 	vchar_t *vid_natt = NULL;
 #endif
+#ifdef ENABLE_DPD
+	vchar_t *vid_dpd = NULL;
+#endif
 
 	/* validity check */
 	if (iph1->status != PHASE1ST_MSG1RECEIVED) {
@@ -936,6 +945,14 @@ ident_r1send(iph1, msg)
 	if (vid_natt)
 		plist = isakmp_plist_append(plist, vid_natt, ISAKMP_NPTYPE_VID);
 #endif
+#ifdef ENABLE_DPD
+	/* XXX only send DPD VID if remote sent it ? */
+	if(iph1->rmconf->dpd){
+		vid_dpd = set_vendorid(VENDORID_DPD);
+		if (vid_dpd != NULL)
+			plist = isakmp_plist_append(plist, vid_dpd, ISAKMP_NPTYPE_VID);
+	}
+#endif
 
 	iph1->sendbuf = isakmp_plist_set_all (&plist, iph1);
 
@@ -971,6 +988,10 @@ end:
 	if (vid_natt)
 		vfree(vid_natt);
 #endif
+#ifdef ENABLE_DPD
+	if (vid_dpd != NULL)
+		vfree(vid_dpd);
+#endif
 
 	return error;
 }
@@ -1253,7 +1274,6 @@ ident_r3recv(iph1, msg0)
 		case ISAKMP_NPTYPE_HASH:
 			iph1->pl_hash = (struct isakmp_pl_hash *)pa->ptr;
 			break;
-#ifdef HAVE_SIGNING_C
 		case ISAKMP_NPTYPE_CR:
 			if (oakley_savecr(iph1, pa->ptr) < 0)
 				goto end;
@@ -1266,7 +1286,6 @@ ident_r3recv(iph1, msg0)
 			if (isakmp_p2ph(&iph1->sig_p, pa->ptr) < 0)
 				goto end;
 			break;
-#endif
 #ifdef HAVE_GSSAPI
 		case ISAKMP_NPTYPE_GSS:
 			if (isakmp_p2ph(&gsstoken, pa->ptr) < 0)
@@ -1346,6 +1365,8 @@ ident_r3recv(iph1, msg0)
 				/* msg printed inner oakley_validate_auth() */
 				goto end;
 			}
+			EVT_PUSH(iph1->local, iph1->remote, 
+			    EVTT_PEERPH1AUTH_FAILED, NULL);
 			isakmp_info_send_n1(iph1, type, NULL);
 			goto end;
 		}
@@ -1353,12 +1374,10 @@ ident_r3recv(iph1, msg0)
 	}
 #endif
 
-#ifdef HAVE_SIGNING_C
 	if (oakley_checkcr(iph1) < 0) {
 		/* Ignore this error in order to be interoperability. */
 		;
 	}
-#endif
 
 	/*
 	 * XXX: Should we do compare two addresses, ph1handle's and ID
@@ -1514,7 +1533,6 @@ ident_ir2mx(iph1)
 	vchar_t *natd[2] = { NULL, NULL };
 #endif
 
-#ifdef HAVE_SIGNING_C
 	/* create CR if need */
 	if (iph1->side == RESPONDER
 	 && iph1->rmconf->send_cr
@@ -1528,7 +1546,6 @@ ident_ir2mx(iph1)
 			goto end;
 		}
 	}
-#endif
 
 #ifdef HAVE_GSSAPI
 	if (iph1->approval->authmethod == OAKLEY_ATTR_AUTH_METHOD_GSSAPI_KRB)
@@ -1643,7 +1660,6 @@ ident_ir3mx(iph1)
 		/* create isakmp HASH payload */
 		plist = isakmp_plist_append(plist, iph1->hash, ISAKMP_NPTYPE_HASH);
 		break;
-#ifdef HAVE_SIGNING_C
 	case OAKLEY_ATTR_AUTH_METHOD_DSSSIG:
 	case OAKLEY_ATTR_AUTH_METHOD_RSASIG:
 		if (oakley_getmycert(iph1) < 0)
@@ -1682,7 +1698,6 @@ ident_ir3mx(iph1)
 		if (need_cr)
 			plist = isakmp_plist_append(plist, cr, ISAKMP_NPTYPE_CR);
 		break;
-#endif
 #ifdef HAVE_GSSAPI
 	case OAKLEY_ATTR_AUTH_METHOD_GSSAPI_KRB:
 		if (iph1->hash != NULL) {
