@@ -1,4 +1,4 @@
-/* $Id: oakley.c,v 1.15.2.1 2005/01/30 20:33:46 vanhu Exp $ */
+/* $Id: oakley.c,v 1.15.2.2 2005/03/01 09:51:27 vanhu Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -2790,8 +2790,10 @@ oakley_newiv2(iph1, msgid)
 	plogdump(LLV_DEBUG, newivm->iv->v, newivm->iv->l);
 
 end:
-	if (error && newivm != NULL)
+	if (error && newivm != NULL){
 		oakley_delivm(newivm);
+		newivm=NULL;
+	}
 	if (buf != NULL)
 		vfree(buf);
 	return newivm;
