@@ -1,4 +1,4 @@
-/* $Id: pfkey.c,v 1.27.2.2 2005/01/29 09:36:26 manubsd Exp $ */
+/* $Id: pfkey.c,v 1.27.2.5 2005/04/21 08:57:17 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -980,7 +980,7 @@ pk_sendupdate(iph2)
 {
 	struct saproto *pr;
 	struct sockaddr *src = NULL, *dst = NULL;
-	int e_type, e_keylen, a_type, a_keylen, flags;
+	u_int e_type, e_keylen, a_type, a_keylen, flags;
 	u_int satype, mode;
 	u_int64_t lifebyte = 0;
 	u_int wsize = 4;  /* XXX static size of window */ 
@@ -1270,7 +1270,7 @@ pk_sendadd(iph2)
 {
 	struct saproto *pr;
 	struct sockaddr *src = NULL, *dst = NULL;
-	int e_type, e_keylen, a_type, a_keylen, flags;
+	u_int e_type, e_keylen, a_type, a_keylen, flags;
 	u_int satype, mode;
 	u_int64_t lifebyte = 0;
 	u_int wsize = 4; /* XXX static size of window */ 
