@@ -1,4 +1,4 @@
-/*	$KAME: proposal.c,v 1.49 2003/08/19 09:12:38 itojun Exp $	*/
+/* $Id: proposal.c,v 1.13 2004/09/13 14:09:19 ludvigm Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -37,7 +37,11 @@
 #include <sys/queue.h>
 
 #include <netinet/in.h>
-#include <netinet/ipsec.h>
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else
+#  include <netinet/ipsec.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -221,7 +225,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 			plog(LLV_ERROR, LOCATION, NULL,
 				"long lifetime proposed: "
 				"my:%d peer:%d\n",
-				pp2->lifetime, pp1->lifetime);
+				(int)pp2->lifetime, (int)pp1->lifetime);
 			goto err;
 		}
 		if (pp1->lifebyte > pp2->lifebyte) {
@@ -254,7 +258,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 			plog(LLV_NOTIFY, LOCATION, NULL,
 				"use own lifetime: "
 				"my:%d peer:%d\n",
-				pp2->lifetime, pp1->lifetime);
+				(int)pp2->lifetime, (int)pp1->lifetime);
 		}
 
 		/* lifebyte */
@@ -275,7 +279,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 			plog(LLV_ERROR, LOCATION, NULL,
 				"lifetime mismatched: "
 				"my:%d peer:%d\n",
-				pp2->lifetime, pp1->lifetime);
+				(int)pp2->lifetime, (int)pp1->lifetime);
 			goto err;
 		}
 		if (pp1->lifebyte != pp2->lifebyte) {
@@ -375,7 +379,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 			plog(LLV_ERROR, LOCATION, NULL,
 				"spisize mismatched: "
 				"my:%d peer:%d\n",
-				pr2->spisize, pr1->spisize);
+				(int)pr2->spisize, (int)pr1->spisize);
 			goto err;
 		}
 
@@ -385,8 +389,7 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 		{
 			plog(LLV_INFO, LOCATION, NULL, "Adjusting my encmode %s->%s\n",
 			     s_ipsecdoi_encmode(pr2->encmode),
-			     s_ipsecdoi_encmode(pr2->encmode - 2),
-			     pr2);
+			     s_ipsecdoi_encmode(pr2->encmode - ph1->natt_options->mode_udp_diff));
 			pr2->encmode -= ph1->natt_options->mode_udp_diff;
 			pr2->udp_encap = 1;
 		}
@@ -394,10 +397,11 @@ cmpsaprop_alloc(ph1, pp1, pp2, side)
 		if ((ph1->natt_flags & NAT_DETECTED) &&
 		    natt_udp_encap (pr1->encmode))
 		{
-			plog(LLV_INFO, LOCATION, NULL, "Adjusting peer's encmode %s->%s\n",
+			plog(LLV_INFO, LOCATION, NULL, "Adjusting peer's encmode %s(%d)->%s(%d)\n",
 			     s_ipsecdoi_encmode(pr1->encmode),
-			     s_ipsecdoi_encmode(pr1->encmode - 2),
-			     pr1);
+			     pr1->encmode,
+			     s_ipsecdoi_encmode(pr1->encmode - ph1->natt_options->mode_udp_diff),
+			     pr1->encmode - ph1->natt_options->mode_udp_diff);
 			pr1->encmode -= ph1->natt_options->mode_udp_diff;
 			pr1->udp_encap = 1;
 		}
@@ -497,7 +501,7 @@ cmpsaprop(pp1, pp2)
 	if (pp1->lifetime > pp2->lifetime) {
 		plog(LLV_WARNING, LOCATION, NULL,
 			"less lifetime proposed. mine:%d peer:%d\n",
-			pp1->lifetime, pp2->lifetime);
+			(int)pp1->lifetime, (int)pp2->lifetime);
 		/* FALLTHRU */
 	}
 	if (pp1->lifebyte > pp2->lifebyte) {
@@ -864,11 +868,11 @@ printsaproto(pri, pr)
 		" (proto_id=%s spisize=%d spi=%08lx spi_p=%08lx "
 		"encmode=%s reqid=%d:%d)\n",
 		s_ipsecdoi_proto(pr->proto_id),
-		pr->spisize,
+		(int)pr->spisize,
 		(unsigned long)ntohl(pr->spi),
 		(unsigned long)ntohl(pr->spi_p),
 		s_ipsecdoi_attr_v(IPSECDOI_ATTR_ENC_MODE, pr->encmode),
-		pr->reqid_in, pr->reqid_out);
+		(int)pr->reqid_in, (int)pr->reqid_out);
 
 	for (tr = pr->head; tr; tr = tr->next) {
 		printsatrns(pri, pr->proto_id, tr);
