@@ -1,4 +1,4 @@
-/* $Id: sockmisc.c,v 1.17 2004/11/20 16:16:59 monas Exp $ */
+/* $Id: sockmisc.c,v 1.17.2.1 2005/04/21 08:57:18 monas Exp $ */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -259,7 +259,7 @@ getlocaladdr(remote)
 	struct sockaddr *remote;
 {
 	struct sockaddr *local;
-	int local_len = sizeof(struct sockaddr_storage);
+	u_int local_len = sizeof(struct sockaddr_storage);
 	int s;	/* for dummy connection */
 
 	/* allocate buffer */
@@ -312,12 +312,12 @@ recvfromto(s, buf, buflen, flags, from, fromlen, to, tolen)
 	size_t buflen;
 	int flags;
 	struct sockaddr *from;
-	int *fromlen;
+	u_int *fromlen;
 	struct sockaddr *to;
-	int *tolen;
+	u_int *tolen;
 {
 	int otolen;
-	int len;
+	u_int len;
 	struct sockaddr_storage ss;
 	struct msghdr m;
 	struct cmsghdr *cm;
@@ -458,7 +458,7 @@ sendfromto(s, buf, buflen, src, dst, cnt)
 	struct sockaddr *dst;
 {
 	struct sockaddr_storage ss;
-	int len;
+	u_int len;
 	int i;
 
 	if (src->sa_family != dst->sa_family) {
