@@ -35,7 +35,12 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 
-#include <netinet/ipsec.h>
+#include <netinet/in.h> 
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else 
+#  include <netinet/ipsec.h>
+#endif
 #include <netinet/in.h>
 
 #include <stdio.h>
@@ -61,9 +66,7 @@ struct ksmap {
 	char *(*f) __P((int));
 };
 
-static char *num2str __P((int n));
-
-static char *
+char *
 num2str(n)
 	int n;
 {
@@ -615,6 +618,12 @@ static struct ksmap name_attr_isakmp_method[] = {
 { OAKLEY_ATTR_AUTH_METHOD_EGENC,	"Encryption with El-Gamal",	NULL },
 { OAKLEY_ATTR_AUTH_METHOD_EGREV,	"Revised encryption with El-Gamal",	NULL },
 { OAKLEY_ATTR_AUTH_METHOD_GSSAPI_KRB,	"GSS-API on Kerberos 5", NULL },
+#ifdef ENABLE_HYBRID
+{ OAKLEY_ATTR_AUTH_METHOD_HYBRID_DSS_I,	"Hybrid DSS server",	NULL },
+{ OAKLEY_ATTR_AUTH_METHOD_HYBRID_RSA_I,	"Hybrid RSA server",	NULL },
+{ OAKLEY_ATTR_AUTH_METHOD_HYBRID_DSS_R,	"Hybrid DSS client",	NULL },
+{ OAKLEY_ATTR_AUTH_METHOD_HYBRID_RSA_R,	"Hybrid RSA client",	NULL },
+#endif
 };
 
 char *
@@ -772,7 +781,9 @@ static struct ksmap name_pfkey_type[] = {
 { SADB_X_SPDSETIDX,	"X_SPDSETIDX",	NULL },
 { SADB_X_SPDEXPIRE,	"X_SPDEXPIRE",	NULL },
 { SADB_X_SPDDELETE2,	"X_SPDDELETE2",	NULL },
+#ifdef ENABLE_NATT
 { SADB_X_NAT_T_NEW_MAPPING, "X_NAT_T_NEW_MAPPING", NULL },
+#endif
 };
 
 char *
@@ -811,6 +822,9 @@ s_pfkey_satype(k)
 static struct ksmap name_direction[] = {
 { IPSEC_DIR_INBOUND,	"in",	NULL },
 { IPSEC_DIR_OUTBOUND,	"out",	NULL },
+#ifdef HAVE_POLICY_FWD
+{ IPSEC_DIR_FWD,	"fwd",	NULL },
+#endif
 };
 
 char *
@@ -843,3 +857,73 @@ s_proto(k)
 
 	return num2str(k);
 }
+
+char *
+s_doi(int k)
+{
+  switch (k) {
+    case IPSEC_DOI:
+      return "ipsec_doi";
+    default:
+      return num2str(k);
+  }
+}
+
+char *
+s_etype (int k)
+{
+  switch (k) {
+    case ISAKMP_ETYPE_NONE:
+      return "_none";
+    case ISAKMP_ETYPE_BASE:
+      return "base";
+    case ISAKMP_ETYPE_IDENT:
+      return "main";
+    case ISAKMP_ETYPE_AUTH:
+      return "_auth";
+    case ISAKMP_ETYPE_AGG:
+      return "aggressive";
+    case ISAKMP_ETYPE_INFO:
+      return "_info";
+    case ISAKMP_ETYPE_QUICK:
+      return "_quick";
+    case ISAKMP_ETYPE_NEWGRP:
+      return "_newgrp";
+    case ISAKMP_ETYPE_ACKINFO:
+      return "_ackinfo";
+    default:
+      return num2str(k);
+  }
+}
+
+char *
+s_idtype (int k)
+{
+  switch (k) {
+    case IDTYPE_FQDN:
+      return "fqdn";
+    case IDTYPE_USERFQDN:
+      return "user_fqdn";
+    case IDTYPE_KEYID:
+      return "keyid";
+    case IDTYPE_ADDRESS:
+      return "address";
+    case IDTYPE_ASN1DN:
+      return "asn1dn";
+    default:
+      return num2str(k);
+  }
+}
+
+char *
+s_switch (int k)
+{
+  switch (k) {
+    case FALSE:
+      return "off";
+    case TRUE:
+      return "on";
+    default:
+      return num2str(k);
+  }
+}
