@@ -166,13 +166,21 @@
 /* Copy the first part of user declarations.  */
 #line 32 "parse.y"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 
 #include <netinet/in.h>
 #include <net/pfkeyv2.h>
-#include <netinet/ipsec.h>
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else 
+#  include <netinet/ipsec.h>
+#endif
 #include <arpa/inet.h>
 
 #include <string.h>
@@ -193,7 +201,8 @@
 #endif
 
 #define ATOX(c) \
-  (isdigit(c) ? (c - '0') : (isupper(c) ? (c - 'A' + 10) : (c - 'a' + 10)))
+  (isdigit((int)c) ? (c - '0') : \
+    (isupper((int)c) ? (c - 'A' + 10) : (c - 'a' + 10)))
 
 u_int32_t p_spi;
 u_int p_ext, p_alg_enc, p_alg_auth, p_replay, p_mode;
@@ -203,6 +212,11 @@ caddr_t p_key_enc, p_key_auth;
 time_t p_lt_hard, p_lt_soft;
 size_t p_lb_hard, p_lb_soft;
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+extern int last_msg_type;
+extern u_int32_t last_priority;
+#endif
+
 extern int exit_now;
 
 static u_int p_natt_type;
@@ -230,6 +244,7 @@ extern int sendkeymsg __P((char *, size_t));
 extern int yylex __P((void));
 extern void yyfatal __P((const char *));
 extern void yyerror __P((const char *));
+extern int f_rfcmode;
 
 
 /* Enabling traces.  */
@@ -246,7 +261,7 @@ extern void yyerror __P((const char *));
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 99 "parse.y"
+#line 114 "parse.y"
 typedef union YYSTYPE {
 	int num;
 	unsigned long ulnum;
@@ -254,7 +269,7 @@ typedef union YYSTYPE {
 	struct addrinfo *res;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 257 "parse.c"
+#line 272 "parse.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -266,7 +281,7 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 269 "parse.c"
+#line 284 "parse.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -469,15 +484,15 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short yyrline[] =
 {
-       0,   139,   139,   141,   149,   150,   151,   152,   153,   154,
-     155,   156,   157,   158,   159,   165,   177,   196,   208,   223,
-     233,   243,   246,   254,   262,   266,   273,   283,   284,   305,
-     306,   307,   311,   312,   316,   320,   328,   340,   355,   370,
-     386,   407,   431,   446,   459,   463,   492,   494,   498,   499,
-     500,   501,   502,   503,   512,   513,   514,   515,   521,   556,
-     568,   606,   617,   626,   628,   632,   657,   668,   676,   688,
-     689,   694,   702,   711,   722,   729,   730,   731,   754,   758,
-     770,   786,   791
+       0,   154,   154,   156,   164,   165,   166,   167,   168,   169,
+     170,   171,   172,   173,   174,   180,   192,   211,   223,   238,
+     248,   258,   261,   269,   277,   281,   288,   298,   299,   320,
+     321,   322,   326,   327,   331,   335,   343,   355,   370,   385,
+     401,   422,   446,   461,   474,   478,   507,   509,   513,   514,
+     515,   516,   517,   518,   527,   528,   529,   530,   536,   575,
+     587,   625,   636,   645,   647,   651,   676,   687,   695,   707,
+     708,   713,   721,   730,   741,   748,   749,   750,   773,   777,
+     789,   813,   818
 };
 #endif
 
@@ -1285,7 +1300,7 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 142 "parse.y"
+#line 157 "parse.y"
     {
 			free_buffer();
 			parse_init();
@@ -1293,7 +1308,7 @@ yyreduce:
     break;
 
   case 15:
-#line 166 "parse.y"
+#line 181 "parse.y"
     {
 			int status;
 
@@ -1304,7 +1319,7 @@ yyreduce:
     break;
 
   case 16:
-#line 178 "parse.y"
+#line 193 "parse.y"
     {
 			int status;
 
@@ -1322,7 +1337,7 @@ yyreduce:
     break;
 
   case 17:
-#line 197 "parse.y"
+#line 212 "parse.y"
     {
 			int status;
 
@@ -1333,7 +1348,7 @@ yyreduce:
     break;
 
   case 18:
-#line 209 "parse.y"
+#line 224 "parse.y"
     {
 			int status;
 
@@ -1347,7 +1362,7 @@ yyreduce:
     break;
 
   case 19:
-#line 224 "parse.y"
+#line 239 "parse.y"
     {
 			struct sadb_msg msg;
 			setkeymsg0(&msg, SADB_FLUSH, yyvsp[-1].num, sizeof(msg));
@@ -1356,7 +1371,7 @@ yyreduce:
     break;
 
   case 20:
-#line 234 "parse.y"
+#line 249 "parse.y"
     {
 			struct sadb_msg msg;
 			setkeymsg0(&msg, SADB_DUMP, yyvsp[-1].num, sizeof(msg));
@@ -1365,14 +1380,14 @@ yyreduce:
     break;
 
   case 21:
-#line 243 "parse.y"
+#line 258 "parse.y"
     {
 			yyval.num = SADB_SATYPE_UNSPEC;
 		}
     break;
 
   case 22:
-#line 247 "parse.y"
+#line 262 "parse.y"
     {
 			yyval.num = SADB_SATYPE_ESP;
 			if (yyvsp[0].num == 1)
@@ -1383,7 +1398,7 @@ yyreduce:
     break;
 
   case 23:
-#line 255 "parse.y"
+#line 270 "parse.y"
     {
 			yyval.num = SADB_SATYPE_AH;
 			if (yyvsp[0].num == 1)
@@ -1394,14 +1409,14 @@ yyreduce:
     break;
 
   case 24:
-#line 263 "parse.y"
+#line 278 "parse.y"
     {
 			yyval.num = SADB_X_SATYPE_IPCOMP;
 		}
     break;
 
   case 25:
-#line 267 "parse.y"
+#line 282 "parse.y"
     {
 			yyval.num = SADB_SATYPE_ESP;
 			p_ext &= ~SADB_X_EXT_OLD;
@@ -1411,7 +1426,7 @@ yyreduce:
     break;
 
   case 26:
-#line 274 "parse.y"
+#line 289 "parse.y"
     {
 			yyval.num = SADB_SATYPE_ESP;
 			p_ext &= ~SADB_X_EXT_OLD;
@@ -1421,12 +1436,12 @@ yyreduce:
     break;
 
   case 27:
-#line 283 "parse.y"
+#line 298 "parse.y"
     { p_spi = yyvsp[0].ulnum; }
     break;
 
   case 28:
-#line 285 "parse.y"
+#line 300 "parse.y"
     {
 			char *ep;
 			unsigned long v;
@@ -1447,7 +1462,7 @@ yyreduce:
     break;
 
   case 35:
-#line 321 "parse.y"
+#line 336 "parse.y"
     {
 			if (yyvsp[0].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1458,7 +1473,7 @@ yyreduce:
     break;
 
   case 36:
-#line 329 "parse.y"
+#line 344 "parse.y"
     {
 			if (yyvsp[-1].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1470,7 +1485,7 @@ yyreduce:
     break;
 
   case 37:
-#line 340 "parse.y"
+#line 355 "parse.y"
     {
 			if (yyvsp[0].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1489,7 +1504,7 @@ yyreduce:
     break;
 
   case 38:
-#line 355 "parse.y"
+#line 370 "parse.y"
     {
 			if (yyvsp[-1].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1508,7 +1523,7 @@ yyreduce:
     break;
 
   case 39:
-#line 370 "parse.y"
+#line 385 "parse.y"
     {
 			if (yyvsp[0].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1528,7 +1543,7 @@ yyreduce:
     break;
 
   case 40:
-#line 387 "parse.y"
+#line 402 "parse.y"
     {
 			if (yyvsp[-1].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1552,7 +1567,7 @@ yyreduce:
     break;
 
   case 41:
-#line 408 "parse.y"
+#line 423 "parse.y"
     {
 			if (yyvsp[-1].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1576,7 +1591,7 @@ yyreduce:
     break;
 
   case 42:
-#line 431 "parse.y"
+#line 446 "parse.y"
     {
 			if (yyvsp[-1].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1595,7 +1610,7 @@ yyreduce:
     break;
 
   case 43:
-#line 446 "parse.y"
+#line 461 "parse.y"
     {
 			if (yyvsp[0].num < 0) {
 				yyerror("unsupported algorithm");
@@ -1609,14 +1624,14 @@ yyreduce:
     break;
 
   case 44:
-#line 460 "parse.y"
+#line 475 "parse.y"
     {
 			yyval.val = yyvsp[0].val;
 		}
     break;
 
   case 45:
-#line 464 "parse.y"
+#line 479 "parse.y"
     {
 			caddr_t pp_key;
 			caddr_t bp;
@@ -1646,32 +1661,32 @@ yyreduce:
     break;
 
   case 48:
-#line 498 "parse.y"
+#line 513 "parse.y"
     { p_ext |= yyvsp[0].num; }
     break;
 
   case 49:
-#line 499 "parse.y"
+#line 514 "parse.y"
     { p_ext &= ~SADB_X_EXT_CYCSEQ; }
     break;
 
   case 50:
-#line 500 "parse.y"
+#line 515 "parse.y"
     { p_mode = yyvsp[0].num; }
     break;
 
   case 51:
-#line 501 "parse.y"
+#line 516 "parse.y"
     { p_mode = IPSEC_MODE_ANY; }
     break;
 
   case 52:
-#line 502 "parse.y"
+#line 517 "parse.y"
     { p_reqid = yyvsp[0].ulnum; }
     break;
 
   case 53:
-#line 504 "parse.y"
+#line 519 "parse.y"
     {
 			if ((p_ext & SADB_X_EXT_OLD) != 0) {
 				yyerror("replay prevention cannot be used with "
@@ -1683,31 +1698,35 @@ yyreduce:
     break;
 
   case 54:
-#line 512 "parse.y"
+#line 527 "parse.y"
     { p_lt_hard = yyvsp[0].ulnum; }
     break;
 
   case 55:
-#line 513 "parse.y"
+#line 528 "parse.y"
     { p_lt_soft = yyvsp[0].ulnum; }
     break;
 
   case 56:
-#line 514 "parse.y"
+#line 529 "parse.y"
     { p_lb_hard = yyvsp[0].ulnum; }
     break;
 
   case 57:
-#line 515 "parse.y"
+#line 530 "parse.y"
     { p_lb_soft = yyvsp[0].ulnum; }
     break;
 
   case 58:
-#line 522 "parse.y"
+#line 537 "parse.y"
     {
 			int status;
 			struct addrinfo *src, *dst;
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+			last_msg_type = SADB_X_SPDADD;
+#endif
+
 			/* fixed port fields if ulp is icmpv6 */
 			if (yyvsp[-2].val.buf != NULL) {
 				if (yyvsp[-3].num != IPPROTO_ICMPV6)
@@ -1741,7 +1760,7 @@ yyreduce:
     break;
 
   case 59:
-#line 557 "parse.y"
+#line 576 "parse.y"
     {
 			int status;
 
@@ -1753,7 +1772,7 @@ yyreduce:
     break;
 
   case 60:
-#line 569 "parse.y"
+#line 588 "parse.y"
     {
 			int status;
 			struct addrinfo *src, *dst;
@@ -1791,7 +1810,7 @@ yyreduce:
     break;
 
   case 61:
-#line 607 "parse.y"
+#line 626 "parse.y"
     {
 			struct sadb_msg msg;
 			setkeymsg0(&msg, SADB_X_SPDDUMP, SADB_SATYPE_UNSPEC,
@@ -1801,7 +1820,7 @@ yyreduce:
     break;
 
   case 62:
-#line 618 "parse.y"
+#line 637 "parse.y"
     {
 			struct sadb_msg msg;
 			setkeymsg0(&msg, SADB_X_SPDFLUSH, SADB_SATYPE_UNSPEC,
@@ -1811,7 +1830,7 @@ yyreduce:
     break;
 
   case 65:
-#line 633 "parse.y"
+#line 652 "parse.y"
     {
 			char *p;
 
@@ -1836,7 +1855,7 @@ yyreduce:
     break;
 
   case 66:
-#line 658 "parse.y"
+#line 677 "parse.y"
     {
 			yyval.res = parse_addr(yyvsp[0].val.buf, NULL);
 			if (yyval.res == NULL) {
@@ -1847,7 +1866,7 @@ yyreduce:
     break;
 
   case 67:
-#line 669 "parse.y"
+#line 688 "parse.y"
     {
 			yyval.res = parse_addr(yyvsp[0].val.buf, NULL);
 			if (yyval.res == NULL) {
@@ -1858,7 +1877,7 @@ yyreduce:
     break;
 
   case 68:
-#line 677 "parse.y"
+#line 696 "parse.y"
     {
 			yyval.res = parse_addr(yyvsp[-1].val.buf, yyvsp[0].val.buf);
 			if (yyval.res == NULL) {
@@ -1869,17 +1888,17 @@ yyreduce:
     break;
 
   case 69:
-#line 688 "parse.y"
+#line 707 "parse.y"
     { yyval.num = -1; }
     break;
 
   case 70:
-#line 689 "parse.y"
+#line 708 "parse.y"
     { yyval.num = yyvsp[0].ulnum; }
     break;
 
   case 71:
-#line 694 "parse.y"
+#line 713 "parse.y"
     {
 			yyval.val.buf = strdup("0");
 			if (!yyval.val.buf) {
@@ -1891,7 +1910,7 @@ yyreduce:
     break;
 
   case 72:
-#line 703 "parse.y"
+#line 722 "parse.y"
     {
 			yyval.val.buf = strdup("0");
 			if (!yyval.val.buf) {
@@ -1903,7 +1922,7 @@ yyreduce:
     break;
 
   case 73:
-#line 712 "parse.y"
+#line 731 "parse.y"
     {
 			char buf[20];
 			snprintf(buf, sizeof(buf), "%lu", yyvsp[-1].ulnum);
@@ -1917,24 +1936,24 @@ yyreduce:
     break;
 
   case 74:
-#line 723 "parse.y"
+#line 742 "parse.y"
     {
 			yyval.val = yyvsp[-1].val;
 		}
     break;
 
   case 75:
-#line 729 "parse.y"
+#line 748 "parse.y"
     { yyval.num = yyvsp[0].ulnum; }
     break;
 
   case 76:
-#line 730 "parse.y"
+#line 749 "parse.y"
     { yyval.num = IPSEC_ULPROTO_ANY; }
     break;
 
   case 77:
-#line 732 "parse.y"
+#line 751 "parse.y"
     {
 			struct protoent *ent;
 
@@ -1956,7 +1975,7 @@ yyreduce:
     break;
 
   case 78:
-#line 754 "parse.y"
+#line 773 "parse.y"
     {
 			yyval.val.buf = NULL;
 			yyval.val.len = 0;
@@ -1964,7 +1983,7 @@ yyreduce:
     break;
 
   case 79:
-#line 759 "parse.y"
+#line 778 "parse.y"
     {
 			yyval.val.buf = strdup(yyvsp[0].val.buf);
 			if (!yyval.val.buf) {
@@ -1976,9 +1995,12 @@ yyreduce:
     break;
 
   case 80:
-#line 771 "parse.y"
+#line 790 "parse.y"
     {
 			char *policy;
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+			struct sadb_x_policy *xpl;
+#endif
 
 			policy = ipsec_set_policy(yyvsp[0].val.buf, yyvsp[0].val.len);
 			if (policy == NULL) {
@@ -1988,16 +2010,21 @@ yyreduce:
 
 			yyval.val.buf = policy;
 			yyval.val.len = ipsec_get_policylen(policy);
+
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+			xpl = (struct sadb_x_policy *) yyval.val.buf;
+			last_priority = xpl->sadb_x_policy_priority;
+#endif
 		}
     break;
 
   case 81:
-#line 786 "parse.y"
+#line 813 "parse.y"
     { yyval.val = yyvsp[0].val; }
     break;
 
   case 82:
-#line 792 "parse.y"
+#line 819 "parse.y"
     {
 			exit_now = 1;
 			YYACCEPT;
@@ -2008,7 +2035,7 @@ yyreduce:
     }
 
 /* Line 991 of yacc.c.  */
-#line 2011 "parse.c"
+#line 2038 "parse.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -2218,7 +2245,7 @@ yyreturn:
 }
 
 
-#line 797 "parse.y"
+#line 824 "parse.y"
 
 
 int
@@ -2260,6 +2287,11 @@ setkeymsg_spdaddr(type, upper, policy, srcs, splen, dsts, dplen)
 	int plen;
 	struct sockaddr *sa;
 	int salen;
+	struct sadb_x_policy *sp;
+#ifdef HAVE_POLICY_FWD
+	struct sadb_x_ipsecrequest *ps = NULL;
+	int saved_level, saved_id = 0;
+#endif
 
 	msg = (struct sadb_msg *)buf;
 
@@ -2270,6 +2302,7 @@ setkeymsg_spdaddr(type, upper, policy, srcs, splen, dsts, dplen)
 	setkeymsg0(msg, type, SADB_SATYPE_UNSPEC, 0);
 	l = sizeof(struct sadb_msg);
 
+	sp = (struct sadb_x_policy*) (buf + l);
 	memcpy(buf + l, policy->buf, policy->len);
 	l += policy->len;
 
@@ -2329,6 +2362,33 @@ setkeymsg_spdaddr(type, upper, policy, srcs, splen, dsts, dplen)
 
 			sendkeymsg(buf, l);
 
+#ifdef HAVE_POLICY_FWD
+			/* create extra call for FWD policy */
+			if (f_rfcmode && sp->sadb_x_policy_dir == IPSEC_DIR_INBOUND) {
+				sp->sadb_x_policy_dir = IPSEC_DIR_FWD;
+				ps = (struct sadb_x_ipsecrequest*) (sp+1);
+
+				/* if request level is unique, change it to
+				 * require for fwd policy */
+				/* XXX: currently, only first policy is updated
+				 * only. Update following too... */
+				saved_level = ps->sadb_x_ipsecrequest_level;
+				if (saved_level == IPSEC_LEVEL_UNIQUE) {
+					saved_id = ps->sadb_x_ipsecrequest_reqid;
+					ps->sadb_x_ipsecrequest_reqid=0;
+					ps->sadb_x_ipsecrequest_level=IPSEC_LEVEL_REQUIRE;
+				}
+
+				sendkeymsg(buf, l);
+				/* restoring for next message */
+				sp->sadb_x_policy_dir = IPSEC_DIR_INBOUND;
+				if (saved_level == IPSEC_LEVEL_UNIQUE) {
+					ps->sadb_x_ipsecrequest_reqid = saved_id;
+					ps->sadb_x_ipsecrequest_level = saved_level;
+				}
+			}
+#endif
+
 			n++;
 		}
 	}
@@ -2502,10 +2562,10 @@ setkeymsg_addr(type, satype, srcs, dsts, no_spi)
 }
 
 #ifdef SADB_X_EXT_NAT_T_TYPE
-static __u16 get_port (struct addrinfo *addr)
+static u_int16_t get_port (struct addrinfo *addr)
 {
 	struct sockaddr *s = addr->ai_addr;
-	__u16 port = 0;
+	u_int16_t port = 0;
 
 	switch (s->sa_family) {
 	case AF_INET:
@@ -2802,15 +2862,13 @@ static int
 fix_portstr(spec, sport, dport)
 	vchar_t *spec, *sport, *dport;
 {
-	char *p, *p2;
+	char *p, *p2 = "0";
 	u_int l;
 
 	l = 0;
 	for (p = spec->buf; *p != ',' && *p != '\0' && l < spec->len; p++, l++)
 		;
-	if (*p == '\0') {
-		p2 = "0";
-	} else {
+	if (*p != '\0') {
 		if (*p == ',') {
 			*p = '\0';
 			p2 = ++p;
