@@ -42,7 +42,11 @@
 #include <err.h>
 #include <netinet/in.h>
 #include <net/pfkeyv2.h>
-#include <netinet/ipsec.h>
+#ifdef HAVE_NETINET6_IPSEC
+#  include <netinet6/ipsec.h>
+#else 
+#  include <netinet/ipsec.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -58,8 +62,10 @@
 
 #ifdef HAVE_READLINE
 #include <readline/readline.h>
+#include <readline/history.h>
 #endif
 
+#include "config.h"
 #include "libpfkey.h"
 #include "../../package_version.h"
 
@@ -72,6 +78,7 @@ void sendkeyshort __P((u_int));
 void promisc __P((void));
 int sendkeymsg __P((char *, size_t));
 int postproc __P((struct sadb_msg *, int));
+int verifypriority __P((struct sadb_msg *m));
 int fileproc __P((const char *));
 const char *numstr __P((int));
 void shortdump_hdr __P((void));
@@ -80,6 +87,8 @@ static void printdate __P((void));
 static int32_t gmt2local __P((time_t));
 void stdin_loop __P((void));
 
+extern void parse_init __P((void));
+
 #define MODE_SCRIPT	1
 #define MODE_CMDDUMP	2
 #define MODE_CMDFLUSH	3
@@ -96,9 +105,35 @@ int f_cmddump = 0;
 int f_policy = 0;
 int f_hexdump = 0;
 int f_tflag = 0;
+int f_notreally = 0;
+#ifdef HAVE_POLICY_FWD
+int f_rfcmode = 1;
+#define RK_OPTS "rk"
+#else
+int f_rkwarn = 0;
+#define RK_OPTS ""
+static void
+rkwarn()
+{
+	if (!f_rkwarn) {
+		f_rkwarn = 1;
+		printf("warning: -r and -k options are not supported in this environment\n");
+	}
+}
+
+#endif
 static time_t thiszone;
 
 extern int lineno;
+
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+extern int last_msg_type;
+int last_msg_type = -1;
+
+extern u_int32_t last_priority;
+u_int32_t last_priority = 0;
+#endif
+
 extern int exit_now;
 int exit_now = 0;
 
@@ -110,10 +145,10 @@ usage(int only_version)
 {
 	printf("setkey @(#) %s (%s)\n", TOP_PACKAGE_STRING, TOP_PACKAGE_URL); 
 	if (! only_version) {
-		printf("usage: setkey [-v] file ...\n");
-		printf("       setkey [-v] -c\n");
-		printf("       setkey [-v] -f filename\n");
-		printf("       setkey [-Palv] -D\n");
+		printf("usage: setkey [-v" RK_OPTS "] file ...\n");
+		printf("       setkey [-nv" RK_OPTS "] -c\n");
+		printf("       setkey [-nv" RK_OPTS "] -f filename\n");
+		printf("       setkey [-Palv" RK_OPTS "] -D\n");
 		printf("       setkey [-Pv] -F\n");
 		printf("       setkey [-H] -x\n");
 		printf("       setkey [-V] [-h]\n");
@@ -138,7 +173,7 @@ main(argc, argv)
 
 	thiszone = gmt2local(0);
 
-	while ((c = getopt(argc, argv, "acdf:HlvxDFPhV")) != -1) {
+	while ((c = getopt(argc, argv, "acdf:HlnvxDFPhVrk")) != -1) {
 		switch (c) {
 		case 'c':
 			f_mode = MODE_STDIN;
@@ -162,6 +197,9 @@ main(argc, argv)
 		case 'l':
 			f_forever = 1;
 			break;
+		case 'n':
+			f_notreally = 1;
+			break;
 		case 'H':
 			f_hexdump = 1;
 			break;
@@ -175,6 +213,20 @@ main(argc, argv)
 		case 'v':
 			f_verbose = 1;
 			break;
+		case 'r':
+#ifdef HAVE_POLICY_FWD
+			f_rfcmode = 1;
+#else
+			rkwarn();
+#endif
+			break;
+		case 'k':
+#ifdef HAVE_POLICY_FWD
+			f_rfcmode = 0;
+#else
+			rkwarn();
+#endif
+			break;
 		case 'V':
 			usage(1);
 			break;
@@ -253,7 +305,7 @@ get_supported()
 void
 stdin_loop()
 {
-	char line[1024], *semicolon;
+	char line[1024], *semicolon, *comment;
 	size_t linelen = 0;
 	
 	memset (line, 0, sizeof(line));
@@ -269,11 +321,18 @@ stdin_loop()
 		char read[1024];
 		read[0] = '\0';
 		fgets (read, sizeof(read), stdin);
-		if (read[strlen(read)-1] == '\n')
-			read[strlen(read)-1] = '\0';
 		if (! read[0])
 			break;
+		if (read[strlen(read)-1] == '\n')
+			read[strlen(read)-1] = '\0';
 #endif
+		comment = strchr(read, '#');
+		if (comment)
+			*comment = '\0';
+
+		if (! read[0])
+			continue;
+
 		linelen += snprintf (&line[linelen], sizeof(line) - linelen,
 				     "%s%s", linelen > 0 ? " " : "", read);
 
@@ -284,6 +343,11 @@ stdin_loop()
 #ifdef HAVE_READLINE
 			add_history (line);
 #endif
+
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+			last_msg_type = -1;  /* invalid message type */
+#endif
+
 			parse_string (line);
 			if (exit_now)
 				return;
@@ -396,6 +460,10 @@ sendkeymsg(buf, len)
 	ssize_t l;
 	struct sadb_msg *msg;
 
+	if (f_notreally) {
+		goto end;
+	}
+
     {
 	struct timeval tv;
 	tv.tv_sec = 1;
@@ -466,6 +534,9 @@ postproc(msg, len)
 	struct sadb_msg *msg;
 	int len;
 {
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	static int priority_support_check = 0;
+#endif
 
 	if (msg->sadb_msg_errno != 0) {
 		char inf[80];
@@ -542,11 +613,54 @@ postproc(msg, len)
 			printf("\n");
 		}
 		break;
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+	case SADB_X_SPDADD:
+		if (last_msg_type == SADB_X_SPDADD && last_priority != 0 && 
+		    msg->sadb_msg_pid == getpid() && !priority_support_check) {
+			priority_support_check = 1;	
+			if (!verifypriority(msg))
+				printf ("WARNING: Kernel does not support policy priorities\n");
+		}
+		break;
+#endif
 	}
 
 	return (0);
 }
 
+#ifdef HAVE_PFKEY_POLICY_PRIORITY
+int
+verifypriority(m)
+	struct sadb_msg *m;
+{
+	caddr_t mhp[SADB_EXT_MAX + 1];
+	struct sadb_x_policy *xpl;
+
+	/* check pfkey message. */
+	if (pfkey_align(m, mhp)) {
+		printf("(%s\n", ipsec_strerror());
+		return 0;
+	}
+	if (pfkey_check(mhp)) {
+		printf("%s\n", ipsec_strerror());
+		return 0;
+	}
+
+	xpl = (struct sadb_x_policy *) mhp[SADB_X_EXT_POLICY];
+
+	if (xpl == NULL) {
+		printf("no X_POLICY extension.\n");
+		return 0;
+	}
+
+	/* now make sure they match */
+	if (last_priority != xpl->sadb_x_policy_priority)
+		return 0;
+
+	return 1; 
+}
+#endif
+
 int
 fileproc(filename)
 	const char *filename;
